@@ -1,7 +1,7 @@
 /*
  *******************************************************************************
  *
- *   Copyright (C) 2003, International Business Machines
+ *   Copyright (C) 2003-2013, International Business Machines
  *   Corporation and others.  All Rights Reserved.
  *
  *******************************************************************************
@@ -31,6 +31,10 @@
 #include "uhash.h"
 #include "cstring.h"
 #include "udataswp.h"
+#include "ucln_cmn.h"
+#include "ubidi_props.h"
+
+U_NAMESPACE_USE
 
 U_CDECL_BEGIN
 
@@ -38,12 +42,36 @@ U_CDECL_BEGIN
 Static cache for already opened StringPrep profiles
 */
 static UHashtable *SHARED_DATA_HASHTABLE = NULL;
+static icu::UInitOnce gSharedDataInitOnce;
 
-static UMTX usprepMutex = NULL;
+static UMutex usprepMutex = U_MUTEX_INITIALIZER;
 
+/* format version of spp file */
+//static uint8_t formatVersion[4]={ 0, 0, 0, 0 };
+
+/* the Unicode version of the sprep data */
+static UVersionInfo dataVersion={ 0, 0, 0, 0 };
+
+/* Profile names must be aligned to UStringPrepProfileType */
+static const char * const PROFILE_NAMES[] = {
+    "rfc3491",      /* USPREP_RFC3491_NAMEPREP */
+    "rfc3530cs",    /* USPREP_RFC3530_NFS4_CS_PREP */
+    "rfc3530csci",  /* USPREP_RFC3530_NFS4_CS_PREP_CI */
+    "rfc3491",      /* USPREP_RFC3530_NSF4_CIS_PREP */
+    "rfc3530mixp",  /* USPREP_RFC3530_NSF4_MIXED_PREP_PREFIX */
+    "rfc3491",      /* USPREP_RFC3530_NSF4_MIXED_PREP_SUFFIX */
+    "rfc3722",      /* USPREP_RFC3722_ISCSI */
+    "rfc3920node",  /* USPREP_RFC3920_NODEPREP */
+    "rfc3920res",   /* USPREP_RFC3920_RESOURCEPREP */
+    "rfc4011",      /* USPREP_RFC4011_MIB */
+    "rfc4013",      /* USPREP_RFC4013_SASLPREP */
+    "rfc4505",      /* USPREP_RFC4505_TRACE */
+    "rfc4518",      /* USPREP_RFC4518_LDAP */
+    "rfc4518ci",    /* USPREP_RFC4518_LDAP_CI */
+};
 
 static UBool U_CALLCONV
-isAcceptable(void * /* context */,
+isSPrepAcceptable(void * /* context */,
              const char * /* type */, 
              const char * /* name */,
              const UDataInfo *pInfo) {
@@ -59,6 +87,8 @@ isAcceptable(void * /* context */,
         pInfo->formatVersion[2]==UTRIE_SHIFT &&
         pInfo->formatVersion[3]==UTRIE_INDEX_SHIFT
     ) {
+        //uprv_memcpy(formatVersion, pInfo->formatVersion, 4);
+        uprv_memcpy(dataVersion, pInfo->dataVersion, 4);
         return TRUE;
     } else {
         return FALSE;
@@ -66,14 +96,14 @@ isAcceptable(void * /* context */,
 }
 
 static int32_t U_CALLCONV
-getFoldingOffset(uint32_t data) {
+getSPrepFoldingOffset(uint32_t data) {
 
     return (int32_t)data;
 
 }
 
 /* hashes an entry  */
-static int32_t U_EXPORT2 U_CALLCONV 
+static int32_t U_CALLCONV 
 hashEntry(const UHashTok parm) {
     UStringPrepKey *b = (UStringPrepKey *)parm.pointer;
     UHashTok namekey, pathkey;
@@ -83,7 +113,7 @@ hashEntry(const UHashTok parm) {
 }
 
 /* compares two entries */
-static UBool U_EXPORT2 U_CALLCONV 
+static UBool U_CALLCONV 
 compareEntries(const UHashTok p1, const UHashTok p2) {
     UStringPrepKey *b1 = (UStringPrepKey *)p1.pointer;
     UStringPrepKey *b2 = (UStringPrepKey *)p2.pointer;
@@ -96,242 +126,11 @@ compareEntries(const UHashTok p1, const UHashTok p2) {
         uhash_compareChars(path1, path2)));
 }
 
-U_CDECL_END
-
-U_CFUNC void 
-usprep_init() {
-    umtx_init(&usprepMutex);
-}
-
-/** Initializes the cache for resources */
-static void 
-initCache(UErrorCode *status) {
-  UBool makeCache = FALSE;
-  umtx_lock(&usprepMutex);
-  makeCache = (SHARED_DATA_HASHTABLE ==  NULL);
-  umtx_unlock(&usprepMutex);
-  if(makeCache) {
-      UHashtable *newCache = uhash_open(hashEntry, compareEntries, status);
-      if (U_FAILURE(*status)) {
-          return;
-      }
-      umtx_lock(&usprepMutex);
-      if(SHARED_DATA_HASHTABLE == NULL) {
-          SHARED_DATA_HASHTABLE = newCache;
-          newCache = NULL;
-      }
-      umtx_unlock(&usprepMutex);
-      if(newCache != NULL) {
-          uhash_close(newCache);
-      }
-  }
-}
-
-static UBool U_CALLCONV
-loadData(UStringPrepProfile* profile, 
-         const char* path, 
-         const char* name, 
-         const char* type, 
-         UErrorCode* errorCode) {
-    /* load Unicode SPREP data from file */    
-    UTrie _sprepTrie={ 0,0,0,0,0,0,0 };
-    UDataMemory *dataMemory;
-    const int32_t *p=NULL;
-    const uint8_t *pb;
-    UVersionInfo unicodeVersion;
-    int32_t normVer, uniVer;
-
-    if(errorCode==NULL || U_FAILURE(*errorCode)) {
-        return 0;
-    }
-
-    /* open the data outside the mutex block */
-    //TODO: change the path
-    dataMemory=udata_openChoice(path, type, name, isAcceptable, NULL, errorCode);
-    if(U_FAILURE(*errorCode)) {
-        return FALSE;
-    }
-
-    p=(const int32_t *)udata_getMemory(dataMemory);
-    pb=(const uint8_t *)(p+_SPREP_INDEX_TOP);
-    utrie_unserialize(&_sprepTrie, pb, p[_SPREP_INDEX_TRIE_SIZE], errorCode);
-    _sprepTrie.getFoldingOffset=getFoldingOffset;
-
-
-    if(U_FAILURE(*errorCode)) {
-        udata_close(dataMemory);
-        return FALSE;
-    }
-
-    /* in the mutex block, set the data for this process */
-    umtx_lock(&usprepMutex);
-    if(profile->sprepData==NULL) {
-        profile->sprepData=dataMemory;
-        dataMemory=NULL;
-        uprv_memcpy(&profile->indexes, p, sizeof(profile->indexes));
-        uprv_memcpy(&profile->sprepTrie, &_sprepTrie, sizeof(UTrie));
-    } else {
-        p=(const int32_t *)udata_getMemory(profile->sprepData);
-    }
-    umtx_unlock(&usprepMutex);
-    /* initialize some variables */
-    profile->mappingData=(uint16_t *)((uint8_t *)(p+_SPREP_INDEX_TOP)+profile->indexes[_SPREP_INDEX_TRIE_SIZE]);
-    
-    /* 
-     * check the normalization corrections version and the current Unicode version 
-     * supported by ICU 
-     */
-    u_versionFromString(unicodeVersion, U_UNICODE_VERSION);
-    normVer = profile->indexes[_SPREP_NORM_CORRECTNS_LAST_UNI_VERSION];
-    uniVer  = (unicodeVersion[0] << 24) + (unicodeVersion[1] << 16) + 
-              (unicodeVersion[2] << 8 ) + (unicodeVersion[3]);
-
-    if( normVer < uniVer &&
-        ((profile->indexes[_SPREP_OPTIONS] & _SPREP_NORMALIZATION_ON) > 0) /* normalization turned on*/
-      ){
-        *errorCode = U_INVALID_FORMAT_ERROR;
-        udata_close(dataMemory);
-        return FALSE;
-    }
-    profile->isDataLoaded = TRUE;
-
-    /* if a different thread set it first, then close the extra data */
-    if(dataMemory!=NULL) {
-        udata_close(dataMemory); /* NULL if it was set correctly */
-    }
-
-
-    return profile->isDataLoaded;
-}
-
-static UStringPrepProfile* 
-usprep_getProfile(const char* path, 
-                  const char* name,
-                  UErrorCode *status){
-
-    UStringPrepProfile* profile = NULL;
-
-    initCache(status);
-
-    if(U_FAILURE(*status)){
-        return NULL;
-    }
-
-    UStringPrepKey stackKey;
-    /* 
-     * const is cast way to save malloc, strcpy and free calls 
-     * we use the passed in pointers for fetching the data from the 
-     * hash table which is safe
-     */
-    stackKey.name = (char*) name;
-    stackKey.path = (char*) path;
-
-    /* fetch the data from the cache */
-    profile = (UStringPrepProfile*) (uhash_get(SHARED_DATA_HASHTABLE,&stackKey));
-    
-    if(profile == NULL){
-        UStringPrepKey* key   = (UStringPrepKey*) uprv_malloc(sizeof(UStringPrepKey));
-        if(key == NULL){
-            *status = U_MEMORY_ALLOCATION_ERROR;
-            return NULL;
-        }
-        /* else load the data and put the data in the cache */
-        profile = (UStringPrepProfile*) uprv_malloc(sizeof(UStringPrepProfile));
-        if(profile == NULL){
-            *status = U_MEMORY_ALLOCATION_ERROR;
-            uprv_free(key);
-            return NULL;
-        }
-
-        /* initialize the data struct members */
-        uprv_memset(profile->indexes,0,sizeof(profile->indexes));
-        profile->mappingData = NULL;
-        profile->sprepData   = NULL;
-        profile->refCount    = 0;
-    
-        /* initialize the  key memebers */
-        key->name  = (char*) uprv_malloc(uprv_strlen(name)+1);
-        if(key->name == NULL){
-            *status = U_MEMORY_ALLOCATION_ERROR;
-            uprv_free(key);
-            uprv_free(profile);
-            return NULL;
-        }
-
-        uprv_strcpy(key->name, name);
-        
-        key->path=NULL;
-
-        if(path != NULL){
-            key->path      = (char*) uprv_malloc(uprv_strlen(path)+1);
-            if(key->path == NULL){
-                *status = U_MEMORY_ALLOCATION_ERROR;
-                uprv_free(key->path);
-                uprv_free(key);
-                uprv_free(profile);
-                return NULL;
-            }
-            uprv_strcpy(key->path, path);
-        }        
-
-        /* load the data */
-        if(!loadData(profile, path, name, _SPREP_DATA_TYPE, status) || U_FAILURE(*status) ){
-            return NULL;
-        }
-        
-        /* get the options */
-        profile->doNFKC            = (UBool)((profile->indexes[_SPREP_OPTIONS] & _SPREP_NORMALIZATION_ON) > 0);
-        profile->checkBiDi         = (UBool)((profile->indexes[_SPREP_OPTIONS] & _SPREP_CHECK_BIDI_ON) > 0);
-        
-        umtx_lock(&usprepMutex);
-        /* add the data object to the cache */
-        uhash_put(SHARED_DATA_HASHTABLE, key, profile, status);
-        umtx_unlock(&usprepMutex);
-    }
-    umtx_lock(&usprepMutex);
-    /* increment the refcount */
-    profile->refCount++;
-    umtx_unlock(&usprepMutex);
-
-    return profile;
-}
-
-U_CAPI UStringPrepProfile* U_EXPORT2
-usprep_open(const char* path, 
-            const char* name,
-            UErrorCode* status){
-
-    if(status == NULL || U_FAILURE(*status)){
-        return NULL;
-    }
-    /* initialize the mutex */
-    usprep_init();
-       
-    /* initialize the profile struct members */
-    return usprep_getProfile(path,name,status);;
-}
-
-U_CAPI void U_EXPORT2
-usprep_close(UStringPrepProfile* profile){
-    if(profile==NULL){
-        return;
-    }
-
-    umtx_lock(&usprepMutex);
-    /* decrement the ref count*/
-    if(profile->refCount > 0){
-        profile->refCount--;
-    }
-    umtx_unlock(&usprepMutex);
-    
-}
-
 static void 
 usprep_unload(UStringPrepProfile* data){
     udata_close(data->sprepData);
 }
 
-
 static int32_t 
 usprep_internal_flushCache(UBool noRefCount){
     UStringPrepProfile *profile = NULL;
@@ -389,8 +188,7 @@ usprep_flushCache(){
 }
 */
 
-U_CFUNC UBool 
-usprep_cleanup(void){
+static UBool U_CALLCONV usprep_cleanup(void){
     if (SHARED_DATA_HASHTABLE != NULL) {
         usprep_internal_flushCache(TRUE);
         if (SHARED_DATA_HASHTABLE != NULL && uhash_count(SHARED_DATA_HASHTABLE) == 0) {
@@ -398,12 +196,239 @@ usprep_cleanup(void){
             SHARED_DATA_HASHTABLE = NULL;
         }
     }
-
-    umtx_destroy(&usprepMutex);             /* Don't worry about destroying the mutex even  */
-                                            /*  if the hash table still exists.  The mutex  */
-                                            /*  will lazily re-init  itself if needed.      */
+    gSharedDataInitOnce.reset();
     return (SHARED_DATA_HASHTABLE == NULL);
 }
+U_CDECL_END
+
+
+/** Initializes the cache for resources */
+static void U_CALLCONV
+createCache(UErrorCode &status) {
+    SHARED_DATA_HASHTABLE = uhash_open(hashEntry, compareEntries, NULL, &status);
+    if (U_FAILURE(status)) {
+        SHARED_DATA_HASHTABLE = NULL;
+    }
+    ucln_common_registerCleanup(UCLN_COMMON_USPREP, usprep_cleanup);
+}
+
+static void 
+initCache(UErrorCode *status) {
+    umtx_initOnce(gSharedDataInitOnce, &createCache, *status);
+}
+
+static UBool U_CALLCONV
+loadData(UStringPrepProfile* profile, 
+         const char* path, 
+         const char* name, 
+         const char* type, 
+         UErrorCode* errorCode) {
+    /* load Unicode SPREP data from file */    
+    UTrie _sprepTrie={ 0,0,0,0,0,0,0 };
+    UDataMemory *dataMemory;
+    const int32_t *p=NULL;
+    const uint8_t *pb;
+    UVersionInfo normUnicodeVersion;
+    int32_t normUniVer, sprepUniVer, normCorrVer;
+
+    if(errorCode==NULL || U_FAILURE(*errorCode)) {
+        return 0;
+    }
+
+    /* open the data outside the mutex block */
+    //TODO: change the path
+    dataMemory=udata_openChoice(path, type, name, isSPrepAcceptable, NULL, errorCode);
+    if(U_FAILURE(*errorCode)) {
+        return FALSE;
+    }
+
+    p=(const int32_t *)udata_getMemory(dataMemory);
+    pb=(const uint8_t *)(p+_SPREP_INDEX_TOP);
+    utrie_unserialize(&_sprepTrie, pb, p[_SPREP_INDEX_TRIE_SIZE], errorCode);
+    _sprepTrie.getFoldingOffset=getSPrepFoldingOffset;
+
+
+    if(U_FAILURE(*errorCode)) {
+        udata_close(dataMemory);
+        return FALSE;
+    }
+
+    /* in the mutex block, set the data for this process */
+    umtx_lock(&usprepMutex);
+    if(profile->sprepData==NULL) {
+        profile->sprepData=dataMemory;
+        dataMemory=NULL;
+        uprv_memcpy(&profile->indexes, p, sizeof(profile->indexes));
+        uprv_memcpy(&profile->sprepTrie, &_sprepTrie, sizeof(UTrie));
+    } else {
+        p=(const int32_t *)udata_getMemory(profile->sprepData);
+    }
+    umtx_unlock(&usprepMutex);
+    /* initialize some variables */
+    profile->mappingData=(uint16_t *)((uint8_t *)(p+_SPREP_INDEX_TOP)+profile->indexes[_SPREP_INDEX_TRIE_SIZE]);
+    
+    u_getUnicodeVersion(normUnicodeVersion);
+    normUniVer = (normUnicodeVersion[0] << 24) + (normUnicodeVersion[1] << 16) + 
+                 (normUnicodeVersion[2] << 8 ) + (normUnicodeVersion[3]);
+    sprepUniVer = (dataVersion[0] << 24) + (dataVersion[1] << 16) + 
+                  (dataVersion[2] << 8 ) + (dataVersion[3]);
+    normCorrVer = profile->indexes[_SPREP_NORM_CORRECTNS_LAST_UNI_VERSION];
+    
+    if(U_FAILURE(*errorCode)){
+        udata_close(dataMemory);
+        return FALSE;
+    }
+    if( normUniVer < sprepUniVer && /* the Unicode version of SPREP file must be less than the Unicode Vesion of the normalization data */
+        normUniVer < normCorrVer && /* the Unicode version of the NormalizationCorrections.txt file should be less than the Unicode Vesion of the normalization data */
+        ((profile->indexes[_SPREP_OPTIONS] & _SPREP_NORMALIZATION_ON) > 0) /* normalization turned on*/
+      ){
+        *errorCode = U_INVALID_FORMAT_ERROR;
+        udata_close(dataMemory);
+        return FALSE;
+    }
+    profile->isDataLoaded = TRUE;
+
+    /* if a different thread set it first, then close the extra data */
+    if(dataMemory!=NULL) {
+        udata_close(dataMemory); /* NULL if it was set correctly */
+    }
+
+
+    return profile->isDataLoaded;
+}
+
+static UStringPrepProfile* 
+usprep_getProfile(const char* path, 
+                  const char* name,
+                  UErrorCode *status){
+
+    UStringPrepProfile* profile = NULL;
+
+    initCache(status);
+
+    if(U_FAILURE(*status)){
+        return NULL;
+    }
+
+    UStringPrepKey stackKey;
+    /* 
+     * const is cast way to save malloc, strcpy and free calls 
+     * we use the passed in pointers for fetching the data from the 
+     * hash table which is safe
+     */
+    stackKey.name = (char*) name;
+    stackKey.path = (char*) path;
+
+    /* fetch the data from the cache */
+    umtx_lock(&usprepMutex);
+    profile = (UStringPrepProfile*) (uhash_get(SHARED_DATA_HASHTABLE,&stackKey));
+    if(profile != NULL) {
+        profile->refCount++;
+    }
+    umtx_unlock(&usprepMutex);
+    
+    if(profile == NULL) {
+        /* else load the data and put the data in the cache */
+        LocalMemory<UStringPrepProfile> newProfile;
+        if(newProfile.allocateInsteadAndReset() == NULL) {
+            *status = U_MEMORY_ALLOCATION_ERROR;
+            return NULL;
+        }
+
+        /* load the data */
+        if(!loadData(newProfile.getAlias(), path, name, _SPREP_DATA_TYPE, status) || U_FAILURE(*status) ){
+            return NULL;
+        }
+
+        /* get the options */
+        newProfile->doNFKC = (UBool)((newProfile->indexes[_SPREP_OPTIONS] & _SPREP_NORMALIZATION_ON) > 0);
+        newProfile->checkBiDi = (UBool)((newProfile->indexes[_SPREP_OPTIONS] & _SPREP_CHECK_BIDI_ON) > 0);
+
+        if(newProfile->checkBiDi) {
+            newProfile->bdp = ubidi_getSingleton();
+        }
+
+        LocalMemory<UStringPrepKey> key;
+        LocalMemory<char> keyName;
+        LocalMemory<char> keyPath;
+        if( key.allocateInsteadAndReset() == NULL ||
+            keyName.allocateInsteadAndCopy(uprv_strlen(name)+1) == NULL ||
+            (path != NULL &&
+             keyPath.allocateInsteadAndCopy(uprv_strlen(path)+1) == NULL)
+         ) {
+            *status = U_MEMORY_ALLOCATION_ERROR;
+            usprep_unload(newProfile.getAlias());
+            return NULL;
+        }
+
+        umtx_lock(&usprepMutex);
+        // If another thread already inserted the same key/value, refcount and cleanup our thread data
+        profile = (UStringPrepProfile*) (uhash_get(SHARED_DATA_HASHTABLE,&stackKey));
+        if(profile != NULL) {
+            profile->refCount++;
+            usprep_unload(newProfile.getAlias());
+        }
+        else {
+            /* initialize the key members */
+            key->name = keyName.orphan();
+            uprv_strcpy(key->name, name);
+            if(path != NULL){
+                key->path = keyPath.orphan();
+                uprv_strcpy(key->path, path);
+            }        
+            profile = newProfile.orphan();
+    
+            /* add the data object to the cache */
+            profile->refCount = 1;
+            uhash_put(SHARED_DATA_HASHTABLE, key.orphan(), profile, status);
+        }
+        umtx_unlock(&usprepMutex);
+    }
+
+    return profile;
+}
+
+U_CAPI UStringPrepProfile* U_EXPORT2
+usprep_open(const char* path, 
+            const char* name,
+            UErrorCode* status){
+
+    if(status == NULL || U_FAILURE(*status)){
+        return NULL;
+    }
+       
+    /* initialize the profile struct members */
+    return usprep_getProfile(path,name,status);
+}
+
+U_CAPI UStringPrepProfile* U_EXPORT2
+usprep_openByType(UStringPrepProfileType type,
+				  UErrorCode* status) {
+    if(status == NULL || U_FAILURE(*status)){
+        return NULL;
+    }
+    int32_t index = (int32_t)type;
+    if (index < 0 || index >= (int32_t)(sizeof(PROFILE_NAMES)/sizeof(PROFILE_NAMES[0]))) {
+        *status = U_ILLEGAL_ARGUMENT_ERROR;
+        return NULL;
+    }
+    return usprep_open(NULL, PROFILE_NAMES[index], status);
+}
+
+U_CAPI void U_EXPORT2
+usprep_close(UStringPrepProfile* profile){
+    if(profile==NULL){
+        return;
+    }
+
+    umtx_lock(&usprepMutex);
+    /* decrement the ref count*/
+    if(profile->refCount > 0){
+        profile->refCount--;
+    }
+    umtx_unlock(&usprepMutex);
+    
+}
 
 U_CFUNC void 
 uprv_syntaxError(const UChar* rules, 
@@ -413,34 +438,28 @@ uprv_syntaxError(const UChar* rules,
     if(parseError == NULL){
         return;
     }
-    if(pos == rulesLen && rulesLen >0){
-        pos--;
-    }
     parseError->offset = pos;
     parseError->line = 0 ; // we are not using line numbers 
     
     // for pre-context
-    int32_t start = (pos <=U_PARSE_CONTEXT_LEN)? 0 : (pos - (U_PARSE_CONTEXT_LEN-1));
-    int32_t stop  = pos;
+    int32_t start = (pos < U_PARSE_CONTEXT_LEN)? 0 : (pos - (U_PARSE_CONTEXT_LEN-1));
+    int32_t limit = pos;
     
-    u_memcpy(parseError->preContext,rules+start,stop-start);
+    u_memcpy(parseError->preContext,rules+start,limit-start);
     //null terminate the buffer
-    parseError->preContext[stop-start] = 0;
+    parseError->preContext[limit-start] = 0;
     
-    //for post-context
+    // for post-context; include error rules[pos]
     start = pos;
-    if(start<rulesLen) {
-        U16_FWD_1(rules, start, rulesLen);
+    limit = start + (U_PARSE_CONTEXT_LEN-1);
+    if (limit > rulesLen) {
+        limit = rulesLen;
     }
-
-    stop  = ((pos+U_PARSE_CONTEXT_LEN)<= rulesLen )? (pos+(U_PARSE_CONTEXT_LEN)) : 
-                                                            rulesLen;
-    if(start < stop){
-        u_memcpy(parseError->postContext,rules+start,stop-start);
-        //null terminate the buffer
-        parseError->postContext[stop-start]= 0;
+    if (start < rulesLen) {
+        u_memcpy(parseError->postContext,rules+start,limit-start);
     }
-    
+    //null terminate the buffer
+    parseError->postContext[limit-start]= 0;
 }
 
 
@@ -455,8 +474,12 @@ getValues(uint16_t trieWord, int16_t& value, UBool& isIndex){
          * the source codepoint is copied to the destination
          */
         type = USPREP_TYPE_LIMIT;
+        isIndex =FALSE;
+        value = 0;
     }else if(trieWord >= _SPREP_TYPE_THRESHOLD){
         type = (UStringPrepType) (trieWord - _SPREP_TYPE_THRESHOLD);
+        isIndex =FALSE;
+        value = 0;
     }else{
         /* get the type */
         type = USPREP_MAP;
@@ -464,12 +487,10 @@ getValues(uint16_t trieWord, int16_t& value, UBool& isIndex){
         if(trieWord & 0x02){
             isIndex = TRUE;
             value = trieWord  >> 2; //mask off the lower 2 bits and shift
-
         }else{
             isIndex = FALSE;
             value = (int16_t)trieWord;
             value =  (value >> 2);
-
         }
  
         if((trieWord>>2) == _SPREP_MAX_INDEX_VALUE){
@@ -582,8 +603,11 @@ static int32_t
 usprep_normalize(   const UChar* src, int32_t srcLength, 
                     UChar* dest, int32_t destCapacity,
                     UErrorCode* status ){
-
-    return unorm_normalize(src,srcLength,UNORM_NFKC,UNORM_UNICODE_3_2,dest,destCapacity,status);
+    return unorm_normalize(
+        src, srcLength,
+        UNORM_NFKC, UNORM_UNICODE_3_2,
+        dest, destCapacity,
+        status);
 }
 
 
@@ -734,17 +758,19 @@ usprep_prepare(   const UStringPrepProfile* profile,
             goto CLEANUP;
         }
 
-        direction = u_charDirection(ch);
-        if(firstCharDir == U_CHAR_DIRECTION_COUNT){
-            firstCharDir = direction;
-        }
-        if(direction == U_LEFT_TO_RIGHT){
-            leftToRight = TRUE;
-            ltrPos = b2Index-1;
-        }
-        if(direction == U_RIGHT_TO_LEFT || direction == U_RIGHT_TO_LEFT_ARABIC){
-            rightToLeft = TRUE;
-            rtlPos = b2Index-1;
+        if(profile->checkBiDi) {
+            direction = ubidi_getClass(profile->bdp, ch);
+            if(firstCharDir == U_CHAR_DIRECTION_COUNT){
+                firstCharDir = direction;
+            }
+            if(direction == U_LEFT_TO_RIGHT){
+                leftToRight = TRUE;
+                ltrPos = b2Index-1;
+            }
+            if(direction == U_RIGHT_TO_LEFT || direction == U_RIGHT_TO_LEFT_ARABIC){
+                rightToLeft = TRUE;
+                rtlPos = b2Index-1;
+            }
         }
     }           
     if(profile->checkBiDi == TRUE){
@@ -765,7 +791,7 @@ usprep_prepare(   const UStringPrepProfile* profile,
             return FALSE;
         }
     }
-    if(b2Len <= destCapacity){
+    if(b2Len>0 && b2Len <= destCapacity){
         uprv_memmove(dest,b2, b2Len*U_SIZEOF_UCHAR);
     }
 
