@@ -1,6 +1,6 @@
 /********************************************************************
- * COPYRIGHT: 
- * Copyright (c) 1997-2003, International Business Machines Corporation and
+ * COPYRIGHT:
+ * Copyright (c) 1997-2013, International Business Machines Corporation and
  * others. All Rights Reserved.
  ********************************************************************/
 /********************************************************************************
@@ -61,20 +61,20 @@ static  void TestDisplayNames(void);
  **/
  static void TestVariantParsing(void);
 
- /* Make sure that the locale data is good. */
- static void TestLocaleStructure(void);
-
- /* Make sure that Country information is the same across locales, within reason. */
- static void TestConsistentCountryInfo(void);
-
- /* Make sure we can pass "de_DE@Collation=PHONEBOOK" */
- static void MoreVariants(void);
- 
  /* Test getting keyword enumeratin */
  static void TestKeywordVariants(void);
 
+ static void TestKeywordSet(void);
+ static void TestKeywordSetError(void);
+
  /* Test getting keyword values */
  static void TestKeywordVariantParsing(void);
+ 
+ /* Test warning for no data in getDisplay* */
+ static void TestDisplayNameWarning(void);
+
+ /* Test uloc_getLocaleForLCID */
+ static void TestGetLocaleForLCID(void);
 
 /**
  * routine to perform subtests, used by TestDisplayNames
@@ -89,6 +89,8 @@ static  void TestDisplayNames(void);
 
  static void TestGetBaseName(void);
 
+static void TestTrailingNull(void);
+
 static void TestGetLocale(void);
 
 /**
@@ -97,5 +99,33 @@ static void TestGetLocale(void);
 static void setUpDataTable(void);
 static void cleanUpDataTable(void);
 /*static void displayDataTable(void);*/
+static void TestAcceptLanguage(void);
+
+/**
+ * test locale aliases 
+*/
+static void TestCalendar(void); 
+static void TestDateFormat(void);
+static void TestCollation(void);
+static void TestULocale(void);
+static void TestUResourceBundle(void);
+static void TestDisplayName(void);
+
+static void TestAcceptLanguage(void);
+
+static void TestOrientation(void);
+
+static void TestLikelySubtags(void);
+
+/**
+ * lanuage tag
+ */
+static void TestForLanguageTag(void);
+static void TestToLanguageTag(void);
+
+/**
+ * locale data
+ */
+static void TestEnglishExemplarCharacters(void);
 
 #endif
