@@ -1,6 +1,6 @@
 /********************************************************************
  * COPYRIGHT: 
- * Copyright (c) 1997-2001, International Business Machines Corporation and
+ * Copyright (c) 1997-2003, 2007-2009 International Business Machines Corporation and
  * others. All Rights Reserved.
  ********************************************************************/
 /********************************************************************************
@@ -17,6 +17,10 @@
 #ifndef _CNUMDEPTST
 #define _CNUMDEPTST
 
+#include "unicode/utypes.h"
+
+#if !UCONFIG_NO_FORMATTING
+
 #include "cintltst.h"
 
 /* The function used to test differnet format patterns*/
@@ -34,6 +38,9 @@ static void TestCurrencySign(void);
 /* Test proper rounding by the format method.*/
 static void TestRounding487(void);
 
+/* Test proper handling of rounding modes. */
+static void TestRounding5350(void);
+
 /* Test localized currency patterns. */
 static void TestCurrency(void);
 
@@ -44,5 +51,12 @@ static void TestSecondaryGrouping(void);
 
 /*Internal functions used*/
 static void roundingTest(UNumberFormat*, double,  int32_t, const char*);
+static void roundingTest2(UNumberFormat*, double, int32_t, const char*);
+
+static void TestCurrencyKeywords(void);
+
+static void TestGetKeywordValuesForLocale(void);
+
+#endif /* #if !UCONFIG_NO_FORMATTING */
 
 #endif
