@@ -1,6 +1,6 @@
 /********************************************************************
  * COPYRIGHT: 
- * Copyright (c) 1997-2003, International Business Machines Corporation and
+ * Copyright (c) 1997-2013, International Business Machines Corporation and
  * others. All Rights Reserved.
  ********************************************************************/
 /*******************************************************************************
@@ -28,6 +28,8 @@
 #include "crestst.h"
 #include "unicode/ctest.h"
 
+#include "ucol_imp.h" /* collation */
+
 #define LENGTHOF(array) (int32_t)(sizeof(array)/sizeof((array)[0]))
 
 static void TestOpenDirect(void);
@@ -91,15 +93,18 @@ void addResourceBundleTest(TestNode** root);
 
 void addResourceBundleTest(TestNode** root)
 {
+#if !UCONFIG_NO_FILE_IO && !UCONFIG_NO_LEGACY_CONVERSION
     addTest(root, &TestConstruction1, "tsutil/crestst/TestConstruction1");
     addTest(root, &TestOpenDirect, "tsutil/crestst/TestOpenDirect");
     addTest(root, &TestResourceBundles, "tsutil/crestst/TestResourceBundles");
     addTest(root, &TestTable32, "tsutil/crestst/TestTable32");
-    addTest(root, &TestFallback, "tsutil/crestst/TestFallback");
-    addTest(root, &TestAliasConflict, "tsutil/crestst/TestAliasConflict");
     addTest(root, &TestFileStream, "tsutil/crestst/TestFileStream");
     addTest(root, &TestGetSize, "tsutil/crestst/TestGetSize");
     addTest(root, &TestGetLocaleByType, "tsutil/crestst/TestGetLocaleByType");
+#endif
+    addTest(root, &TestFallback, "tsutil/crestst/TestFallback");
+    addTest(root, &TestAliasConflict, "tsutil/crestst/TestAliasConflict");
+
 }
 
 
@@ -114,12 +119,12 @@ void TestAliasConflict(void) {
     he = ures_open(NULL, "he", &status);
     iw = ures_open(NULL, "iw", &status);
     if(U_FAILURE(status)) {
-        log_err("Failed to get resource with %s\n", myErrorName(status));
+        log_err_status(status, "Failed to get resource with %s\n", myErrorName(status));
     }
     ures_close(iw);
-    result = ures_getStringByKey(he, "localPatternChars", &resultLen, &status);
+    result = ures_getStringByKey(he, "ExemplarCharacters", &resultLen, &status);
     if(U_FAILURE(status) || result == NULL) {
-        log_err("Failed to get resource with %s\n", myErrorName(status));
+        log_err_status(status, "Failed to get resource with %s\n", myErrorName(status));
     }
     ures_close(he);
 }
@@ -127,6 +132,13 @@ void TestAliasConflict(void) {
 
 void TestResourceBundles()
 {
+    UErrorCode status = U_ZERO_ERROR;
+    loadTestData(&status);
+    if(U_FAILURE(status)) {
+        log_data_err("Could not load testdata.dat, status = %s\n", u_errorName(status));
+        return;
+    }
+
     testTag("only_in_Root", TRUE, FALSE, FALSE);
     testTag("in_Root_te", TRUE, TRUE, FALSE);
     testTag("in_Root_te_te_IN", TRUE, TRUE, TRUE);
@@ -156,7 +168,7 @@ void TestConstruction1()
     testdatapath=loadTestData(&err);
     if(U_FAILURE(err))
     {
-        log_err("Could not load testdata.dat %s \n",myErrorName(err));
+        log_data_err("Could not load testdata.dat %s \n",myErrorName(err));
         return;
     }
 
@@ -190,7 +202,7 @@ void TestConstruction1()
         int n;
 
         log_err("Construction test failed for ures_open();\n");
-        if(!VERBOSITY)
+        if(!getTestOption(VERBOSITY_OPTION))
             log_info("(run verbose for more information)\n");
 
         log_verbose("\nGot->");
@@ -417,7 +429,7 @@ static void TestFallback()
     fr_FR = ures_open(NULL, "fr_FR", &status);
     if(U_FAILURE(status))
     {
-        log_err("Couldn't open fr_FR - %d\n", status);
+        log_err_status(status, "Couldn't open fr_FR - %s\n", u_errorName(status));
         return;
     }
 
@@ -426,6 +438,7 @@ static void TestFallback()
 
     /* clear it out..  just do some calls to get the gears turning */
     junk = ures_getStringByKey(fr_FR, "LocaleID", &resultLen, &status);
+    (void)junk;    /* Suppress set but not used warning. */
     status = U_ZERO_ERROR;
     junk = ures_getStringByKey(fr_FR, "LocaleString", &resultLen, &status);
     status = U_ZERO_ERROR;
@@ -433,20 +446,20 @@ static void TestFallback()
     status = U_ZERO_ERROR;
 
     /* OK first one. This should be a Default value. */
-    subResource = ures_getByKey(fr_FR, "CurrencyMap", NULL, &status);
+    subResource = ures_getByKey(fr_FR, "layout", NULL, &status);
     if(status != U_USING_DEFAULT_WARNING)
     {
-        log_data_err("Expected U_USING_DEFAULT_ERROR when trying to get LocaleScript from fr_FR, got %s\n",
+        log_data_err("Expected U_USING_DEFAULT_ERROR when trying to get layout from fr_FR, got %s\n",
             u_errorName(status));
     }
     ures_close(subResource);
     status = U_ZERO_ERROR;
 
     /* and this is a Fallback, to fr */
-    junk = ures_getStringByKey(fr_FR, "DayNames", &resultLen, &status);
+    junk = ures_getStringByKey(fr_FR, "ExemplarCharacters", &resultLen, &status);
     if(status != U_USING_FALLBACK_WARNING)
     {
-        log_data_err("Expected U_USING_FALLBACK_ERROR when trying to get DayNames from fr_FR, got %s\n", 
+        log_data_err("Expected U_USING_FALLBACK_ERROR when trying to get ExemplarCharacters from fr_FR, got %s\n", 
             u_errorName(status));
     }
 
@@ -457,7 +470,7 @@ static void TestFallback()
 
 static void
 TestOpenDirect(void) {
-    UResourceBundle *translit_index, *item;
+    UResourceBundle *idna_rules, *casing, *te_IN, *ne, *item;
     UErrorCode errorCode;
 
     /*
@@ -466,19 +479,19 @@ TestOpenDirect(void) {
      * from root or similar
      */
     errorCode=U_ZERO_ERROR;
-    translit_index=ures_openDirect(NULL, "translit_index", &errorCode);
+    idna_rules=ures_openDirect(loadTestData(&errorCode), "idna_rules", &errorCode);
     if(U_FAILURE(errorCode)) {
-        log_err("ures_openDirect(\"translit_index\") failed: %s\n", u_errorName(errorCode));
+        log_data_err("ures_openDirect(\"idna_rules\") failed: %s\n", u_errorName(errorCode));
         return;
     }
 
-    if(0!=uprv_strcmp("translit_index", ures_getLocale(translit_index, &errorCode))) {
-        log_err("ures_openDirect(\"translit_index\").getLocale()!=translit_index\n");
+    if(0!=uprv_strcmp("idna_rules", ures_getLocale(idna_rules, &errorCode))) {
+        log_err("ures_openDirect(\"idna_rules\").getLocale()!=idna_rules\n");
     }
     errorCode=U_ZERO_ERROR;
 
-    /* try an item in translit_index, must work */
-    item=ures_getByKey(translit_index, "RuleBasedTransliteratorIDs", NULL, &errorCode);
+    /* try an item in idna_rules, must work */
+    item=ures_getByKey(idna_rules, "UnassignedSet", NULL, &errorCode);
     if(U_FAILURE(errorCode)) {
         log_err("translit_index.getByKey(local key) failed: %s\n", u_errorName(errorCode));
         errorCode=U_ZERO_ERROR;
@@ -487,36 +500,118 @@ TestOpenDirect(void) {
     }
 
     /* try an item in root, must fail */
-    item=ures_getByKey(translit_index, "Languages", NULL, &errorCode);
+    item=ures_getByKey(idna_rules, "ShortLanguage", NULL, &errorCode);
     if(U_FAILURE(errorCode)) {
         errorCode=U_ZERO_ERROR;
     } else {
-        log_err("translit_index.getByKey(root key) succeeded!\n");
+        log_err("idna_rules.getByKey(root key) succeeded!\n");
         ures_close(item);
     }
-    ures_close(translit_index);
+    ures_close(idna_rules);
 
-    /* now make sure that "translit_index" will not work with ures_open() */
+    /* now make sure that "idna_rules" will not work with ures_open() */
     errorCode=U_ZERO_ERROR;
-    translit_index=ures_open(NULL, "translit_index", &errorCode);
+    idna_rules=ures_open("testdata", "idna_rules", &errorCode);
     if(U_FAILURE(errorCode) || errorCode==U_USING_DEFAULT_WARNING || errorCode==U_USING_FALLBACK_WARNING) {
         /* falling back to default or root is ok */
         errorCode=U_ZERO_ERROR;
-    } else if(0!=uprv_strcmp("translit_INDEX", ures_getLocale(translit_index, &errorCode))) {
+    } else if(0!=uprv_strcmp("idna_rules", ures_getLocale(idna_rules, &errorCode))) {
         /* Opening this file will work in "files mode" on Windows and the Mac,
            which have case insensitive file systems */
-        log_err("ures_open(\"translit_index\") succeeded, should fail! Got: %s\n", u_errorName(errorCode));
+        log_err("ures_open(\"idna_rules\") succeeded, should fail! Got: %s\n", u_errorName(errorCode));
     }
-    ures_close(translit_index);
+    ures_close(idna_rules);
 
     /* ures_openDirect("translit_index_WronG") must fail */
-    translit_index=ures_openDirect(NULL, "translit_index_WronG", &errorCode);
+    idna_rules=ures_openDirect(NULL, "idna_rules_WronG", &errorCode);
     if(U_FAILURE(errorCode)) {
         errorCode=U_ZERO_ERROR;
     } else {
-        log_err("ures_openDirect(\"translit_index_WronG\") succeeded, should fail!\n");
+        log_err("ures_openDirect(\"idna_rules_WronG\") succeeded, should fail!\n");
     }
-    ures_close(translit_index);
+    ures_close(idna_rules);
+
+    errorCode = U_USING_FALLBACK_WARNING;;
+    idna_rules=ures_openDirect("testdata", "idna_rules", &errorCode);
+    if(U_FAILURE(errorCode)) {
+        log_data_err("ures_openDirect(\"idna_rules\") failed when U_USING_FALLBACK_WARNING was set prior to call: %s\n", u_errorName(errorCode));
+        return;
+    }
+    ures_close(idna_rules);
+
+    /*
+     * ICU 3.6 has new resource bundle syntax and data for bundles that do not
+     * participate in locale fallback. Now,
+     * - ures_open() works like ures_openDirect() on a bundle with a top-level
+     *   type of ":table(nofallback)" _if_ the bundle exists
+     * - ures_open() will continue to find a root bundle if the requested one
+     *   does not exist, unlike ures_openDirect()
+     *
+     * Test with a different bundle than above to avoid confusion in the cache.
+     */
+
+    /*
+     * verify that ures_open("casing"), which now has a nofallback declaration,
+     * does not enable fallbacks
+     */
+    errorCode=U_ZERO_ERROR;
+    casing=ures_open("testdata", "casing", &errorCode);
+    if(U_FAILURE(errorCode)) {
+        log_data_err("ures_open(\"casing\") failed: %s\n", u_errorName(errorCode));
+        return;
+    }
+
+    errorCode=U_ZERO_ERROR;
+    item=ures_getByKey(casing, "Info", NULL, &errorCode);
+    if(U_FAILURE(errorCode)) {
+        log_err("casing.getByKey(Info) failed - %s\n", u_errorName(errorCode));
+    } else {
+        ures_close(item);
+    }
+
+    errorCode=U_ZERO_ERROR;
+    item=ures_getByKey(casing, "ShortLanguage", NULL, &errorCode);
+    if(U_SUCCESS(errorCode)) {
+        log_err("casing.getByKey(root key) succeeded despite nofallback declaration - %s\n", u_errorName(errorCode));
+        ures_close(item);
+    }
+    ures_close(casing);
+
+    /*
+     * verify that ures_open("ne") finds the root bundle but
+     * ures_openDirect("ne") does not
+     */
+    errorCode=U_ZERO_ERROR;
+    ne=ures_open("testdata", "ne", &errorCode);
+    if(U_FAILURE(errorCode)) {
+        log_data_err("ures_open(\"ne\") failed (expected to get root): %s\n", u_errorName(errorCode));
+    }
+    if(errorCode!=U_USING_DEFAULT_WARNING || 0!=uprv_strcmp("root", ures_getLocale(ne, &errorCode))) {
+        log_err("ures_open(\"ne\") found something other than \"root\" - %s\n", u_errorName(errorCode));
+    }
+    ures_close(ne);
+
+    errorCode=U_ZERO_ERROR;
+    ne=ures_openDirect("testdata", "ne", &errorCode);
+    if(U_SUCCESS(errorCode)) {
+        log_data_err("ures_openDirect(\"ne\") succeeded unexpectedly\n");
+        ures_close(ne);
+    }
+
+    /* verify that ures_openDirect("te_IN") does not enable fallbacks */
+    errorCode=U_ZERO_ERROR;
+    te_IN=ures_openDirect("testdata", "te_IN", &errorCode);
+    if(U_FAILURE(errorCode)) {
+        log_data_err("ures_open(\"te_IN\") failed: %s\n", u_errorName(errorCode));
+        return;
+    }
+    errorCode=U_ZERO_ERROR;
+    item=ures_getByKey(te_IN, "ShortLanguage", NULL, &errorCode);
+    if(U_SUCCESS(errorCode)) {
+        log_err("te_IN.getByKey(root key) succeeded despite use of ures_openDirect() - %s\n", u_errorName(errorCode));
+        ures_close(item);
+    }
+    ures_close(te_IN);
 }
 
 static int32_t
@@ -685,7 +780,7 @@ static void TestFileStream(void){
     FileStream* pStderr = T_FileStream_stderr();
 
     const char* testline = "This is a test line";
-    int32_t bufLen =uprv_strlen(testline)+10;
+    int32_t bufLen = (int32_t)strlen(testline)+10;
     char* buf = (char*) malloc(bufLen);
     int32_t retLen = 0;
 
@@ -704,52 +799,53 @@ static void TestFileStream(void){
     stream = T_FileStream_open(fileName, "r");
     if(stream==NULL){
         log_data_err("T_FileStream_open failed to open %s\n",fileName);
-    }
-    if(!T_FileStream_file_exists(fileName)){
+    } else {
+      if(!T_FileStream_file_exists(fileName)){
         log_data_err("T_FileStream_file_exists failed to verify existence of %s \n",fileName);
-    }
-
-    retLen=T_FileStream_read(stream,&c,1);
-    if(retLen==0){
+      }
+      
+      retLen=T_FileStream_read(stream,&c,1);
+      if(retLen==0){
         log_data_err("T_FileStream_read failed to read from %s \n",fileName);
-    }
-    retLen=0;
-    T_FileStream_rewind(stream);
-    T_FileStream_read(stream,&c1,1);
-    if(c!=c1){
+      }
+      retLen=0;
+      T_FileStream_rewind(stream);
+      T_FileStream_read(stream,&c1,1);
+      if(c!=c1){
         log_data_err("T_FileStream_rewind failed to rewind %s \n",fileName);
-    }
-    T_FileStream_rewind(stream);
-    c1 = T_FileStream_peek(stream);
-    if(c!=c1){
+      }
+      T_FileStream_rewind(stream);
+      c1 = T_FileStream_peek(stream);
+      if(c!=c1){
         log_data_err("T_FileStream_peek failed to peekd %s \n",fileName);
-    }
-    c = T_FileStream_getc(stream);
-    T_FileStream_ungetc(c,stream);
-    if(c!= T_FileStream_getc(stream)){
+      }
+      c = T_FileStream_getc(stream);
+      T_FileStream_ungetc(c,stream);
+      if(c!= T_FileStream_getc(stream)){
         log_data_err("T_FileStream_ungetc failed to d %s \n",fileName);
-    }
-
-    if(T_FileStream_size(stream)<=0){
+      }
+      
+      if(T_FileStream_size(stream)<=0){
         log_data_err("T_FileStream_size failed to d %s \n",fileName);
-    }
-    if(T_FileStream_error(stream)){
+      }
+      if(T_FileStream_error(stream)){
         log_data_err("T_FileStream_error shouldn't have an error %s\n",fileName);
-    }
-    if(!T_FileStream_error(NULL)){
+      }
+      if(!T_FileStream_error(NULL)){
         log_err("T_FileStream_error didn't get an error %s\n",fileName);
-    }
-    T_FileStream_putc(stream, 0x20);
-    if(!T_FileStream_error(stream)){
+      }
+      T_FileStream_putc(stream, 0x20);
+      if(!T_FileStream_error(stream)){
         /*
-         Warning 
-         writing to a read-only file may not consistently fail on all platforms
-         (e.g. HP-UX, FreeBSD, MacOSX)
+          Warning 
+          writing to a read-only file may not consistently fail on all platforms
+          (e.g. HP-UX, FreeBSD, MacOSX)
         */
         log_verbose("T_FileStream_error didn't get an error when writing to a readonly file %s\n",fileName);
-    }
+      }
 
-    T_FileStream_close(stream);
+      T_FileStream_close(stream);
+    }
     /* test writing function */
     stream=NULL;
     uprv_strcpy(fileName,testdatapath);
@@ -758,32 +854,32 @@ static void TestFileStream(void){
 
     if(stream == NULL){
         log_data_err("Could not open %s for writing\n",fileName);
-    }
-    c= '$';
-    T_FileStream_putc(stream,c);
-    T_FileStream_rewind(stream);
-    if(c != T_FileStream_getc(stream)){
+    } else {
+      c= '$';
+      T_FileStream_putc(stream,c);
+      T_FileStream_rewind(stream);
+      if(c != T_FileStream_getc(stream)){
         log_data_err("T_FileStream_putc failed %s\n",fileName);
-    }
+      }
 
-    T_FileStream_rewind(stream);
-    T_FileStream_writeLine(stream,testline);
-    T_FileStream_rewind(stream);
-    T_FileStream_readLine(stream,buf,bufLen);
-    if(uprv_strncmp(testline, buf,uprv_strlen(buf))!=0){
+      T_FileStream_rewind(stream);
+      T_FileStream_writeLine(stream,testline);
+      T_FileStream_rewind(stream);
+      T_FileStream_readLine(stream,buf,bufLen);
+      if(uprv_strncmp(testline, buf,uprv_strlen(buf))!=0){
         log_data_err("T_FileStream_writeLine failed %s\n",fileName);
-    }
+      }
 
-    T_FileStream_rewind(stream);
-    T_FileStream_write(stream,testline,uprv_strlen(testline));
-    T_FileStream_rewind(stream);
-    retLen = T_FileStream_read(stream, buf, bufLen);
-    if(uprv_strncmp(testline, buf,retLen)!=0){
+      T_FileStream_rewind(stream);
+      T_FileStream_write(stream,testline,(int32_t)strlen(testline));
+      T_FileStream_rewind(stream);
+      retLen = T_FileStream_read(stream, buf, bufLen);
+      if(uprv_strncmp(testline, buf,retLen)!=0){
         log_data_err("T_FileStream_write failed %s\n",fileName);
+      }
+
+      T_FileStream_close(stream);
     }
-
-    T_FileStream_close(stream);
-
     if(!T_FileStream_remove(fileName)){
         log_data_err("T_FileStream_remove failed to delete %s\n",fileName);
     }
@@ -823,7 +919,7 @@ static void TestGetSize(void) {
     
     if(U_FAILURE(status))
     {
-        log_err("Could not load testdata.dat %s\n", u_errorName(status));
+        log_data_err("Could not load testdata.dat %s\n", u_errorName(status));
         return;
     }
     
@@ -881,7 +977,7 @@ static void TestGetLocaleByType(void) {
     
     if(U_FAILURE(status))
     {
-        log_err("Could not load testdata.dat %s\n", u_errorName(status));
+        log_data_err("Could not load testdata.dat %s\n", u_errorName(status));
         return;
     }
     
