@@ -1,7 +1,7 @@
 /*
 *******************************************************************************
 *
-*   Copyright (C) 2002-2006, International Business Machines
+*   Copyright (C) 2002-2013, International Business Machines
 *   Corporation and others.  All Rights Reserved.
 *
 *******************************************************************************
@@ -26,6 +26,8 @@
 #include "cmemory.h"
 #include "cintltst.h"
 
+#define LENGTHOF(array) (int32_t)(sizeof(array)/sizeof((array)[0]))
+
 /* test string case mapping functions --------------------------------------- */
 
 static void
@@ -238,7 +240,7 @@ TestCaseTitle(void) {
     errorCode=U_ZERO_ERROR;
     titleIterChars=ubrk_open(UBRK_CHARACTER, "", beforeTitle, sizeof(beforeTitle)/U_SIZEOF_UCHAR, &errorCode);
     if(U_FAILURE(errorCode)) {
-        log_err("error: ubrk_open(UBRK_CHARACTER)->%s\n", u_errorName(errorCode));
+        log_err_status(errorCode, "error: ubrk_open(UBRK_CHARACTER)->%s\n", u_errorName(errorCode));
         return;
     }
 
@@ -326,6 +328,68 @@ TestCaseTitle(void) {
     ubrk_close(titleIterChars);
 }
 
+static void
+TestCaseDutchTitle(void) {
+    static const UChar
+
+    beforeTitle[]= { 0x69, 0x6A, 0x73, 0x73,  0x45, 0x6c, 0x20, 0x69, 0x67, 0x6c, 0x4f, 0x6f , 0x20 , 0x49, 0x4A, 0x53, 0x53, 0x45, 0x4C },
+    titleRoot[]=   { 0x49, 0x6A, 0x73, 0x73,  0x65, 0x6c, 0x20, 0x49, 0x67, 0x6c, 0x6f, 0x6f , 0x20 , 0x49, 0x6A, 0x73, 0x73, 0x65, 0x6C },
+    titleDutch[]=  { 0x49, 0x4A, 0x73, 0x73,  0x65, 0x6c, 0x20, 0x49, 0x67, 0x6c, 0x6f, 0x6f , 0x20 , 0x49, 0x4A, 0x73, 0x73, 0x65, 0x6C };
+
+    UChar buffer[32];
+    UBreakIterator *titleIterWord;
+    int32_t length;
+    UErrorCode errorCode;
+
+    errorCode=U_ZERO_ERROR;
+    titleIterWord=ubrk_open(UBRK_WORD, "", beforeTitle, sizeof(beforeTitle)/U_SIZEOF_UCHAR, &errorCode);
+    if(U_FAILURE(errorCode)) {
+        log_err_status(errorCode, "error: ubrk_open(UBRK_WORD)->%s\n", u_errorName(errorCode));
+        return;
+    }
+
+    /* titlecase with default locale */
+    buffer[0]=0xabcd;
+    errorCode=U_ZERO_ERROR;
+    length=u_strToTitle(buffer, sizeof(buffer)/U_SIZEOF_UCHAR,
+                        beforeTitle, sizeof(beforeTitle)/U_SIZEOF_UCHAR,
+                        titleIterWord, "",
+                        &errorCode);
+    if( U_FAILURE(errorCode) ||
+        length!=(sizeof(titleRoot)/U_SIZEOF_UCHAR) ||
+        uprv_memcmp(titleRoot, buffer, length*U_SIZEOF_UCHAR)!=0 ||
+        buffer[length]!=0
+    ) {
+        char charsOut[21];
+        u_UCharsToChars(buffer,charsOut,sizeof(charsOut));
+        log_err("error in u_strToTitle(UBRK_CHARACTERS)=%ld error=%s root locale string matches: %s\noutput buffer is {%s}\n",
+            length,
+            u_errorName(errorCode),
+            uprv_memcmp(titleRoot, buffer, length*U_SIZEOF_UCHAR)==0 && buffer[length]==0 ? "yes" : "no", charsOut);
+    }
+    /* titlecase with Dutch locale */
+    buffer[0]=0xabcd;
+    errorCode=U_ZERO_ERROR;
+    length=u_strToTitle(buffer, sizeof(buffer)/U_SIZEOF_UCHAR,
+                        beforeTitle, sizeof(beforeTitle)/U_SIZEOF_UCHAR,
+                        titleIterWord, "nl",
+                        &errorCode);
+    if( U_FAILURE(errorCode) ||
+        length!=(sizeof(titleDutch)/U_SIZEOF_UCHAR) ||
+        uprv_memcmp(titleDutch, buffer, length*U_SIZEOF_UCHAR)!=0 ||
+        buffer[length]!=0
+    ) {
+        char charsOut[21];
+        u_UCharsToChars(buffer,charsOut,sizeof(charsOut));
+        log_err("error in u_strToTitle(UBRK_CHARACTERS)=%ld error=%s dutch locale string matches: %s\noutput buffer is {%s}\n",
+            length,
+            u_errorName(errorCode),
+            uprv_memcmp(titleDutch, buffer, length*U_SIZEOF_UCHAR)==0 && buffer[length]==0 ? "yes" : "no", charsOut);
+    }
+
+    ubrk_close(titleIterWord);
+}
+
 #endif
 
 /* test case folding and case-insensitive string compare -------------------- */
@@ -575,12 +639,14 @@ TestCaseCompare(void) {
 
     lenMixed=u_strlen(mixed);
     lenOtherDefault=u_strlen(otherDefault);
+    (void)lenOtherDefault;    /* Suppress set but not used warning. */
     lenOtherExcludeSpecialI=u_strlen(otherExcludeSpecialI);
     lenDifferent=u_strlen(different);
 
     /* if unicodeVersion()>=3.1 then test exclude-special-i cases as well */
     u_getUnicodeVersion(unicodeVersion);
     isUnicode_3_1= uprv_memcmp(unicodeVersion, unicode_3_1, 4)>=0;
+    (void)isUnicode_3_1;    /* Suppress set but not used warning. */
 
     /* test u_strcasecmp() */
     result=u_strcasecmp(mixed, otherDefault, U_FOLD_CASE_DEFAULT);
@@ -776,19 +842,169 @@ TestUCaseMap(void) {
         log_err("ucasemap_utf8ToUpper(overflow) failed\n");
     }
 
+    /* C API coverage for case folding. More thorough test via C++ intltest's StringCaseTest::TestCasing(). */
+    errorCode=U_ZERO_ERROR;
+    utf8Out[0]=0;
+    length=ucasemap_utf8FoldCase(csm, utf8Out, (int32_t)sizeof(utf8Out), aBc, 3, &errorCode);
+    if(U_FAILURE(errorCode) || length!=3 || 0!=strcmp(abc, utf8Out)) {
+        log_err("ucasemap_utf8FoldCase(aBc) failed\n");
+    }
+
     ucasemap_close(csm);
 }
 
+#if !UCONFIG_NO_BREAK_ITERATION
+
+/* Try titlecasing with options. */
+static void
+TestUCaseMapToTitle(void) {
+    /* "a 'CaT. A 'dOg! 'eTc." where '=U+02BB */
+    /*
+     * Note: The sentence BreakIterator does not recognize a '.'
+     * as a sentence terminator if it is followed by lowercase.
+     * That is why the example has the '!'.
+     */
+    static const UChar
+
+    beforeTitle[]=      { 0x61, 0x20, 0x2bb, 0x43, 0x61, 0x54, 0x2e, 0x20, 0x41, 0x20, 0x2bb, 0x64, 0x4f, 0x67, 0x21, 0x20, 0x2bb, 0x65, 0x54, 0x63, 0x2e },
+    titleWord[]=        { 0x41, 0x20, 0x2bb, 0x43, 0x61, 0x74, 0x2e, 0x20, 0x41, 0x20, 0x2bb, 0x44, 0x6f, 0x67, 0x21, 0x20, 0x2bb, 0x45, 0x74, 0x63, 0x2e },
+    titleWordNoAdjust[]={ 0x41, 0x20, 0x2bb, 0x63, 0x61, 0x74, 0x2e, 0x20, 0x41, 0x20, 0x2bb, 0x64, 0x6f, 0x67, 0x21, 0x20, 0x2bb, 0x65, 0x74, 0x63, 0x2e },
+    titleSentNoLower[]= { 0x41, 0x20, 0x2bb, 0x43, 0x61, 0x54, 0x2e, 0x20, 0x41, 0x20, 0x2bb, 0x64, 0x4f, 0x67, 0x21, 0x20, 0x2bb, 0x45, 0x54, 0x63, 0x2e };
+
+    UChar buffer[32];
+    UCaseMap *csm;
+    UBreakIterator *sentenceIter;
+    const UBreakIterator *iter;
+    int32_t length;
+    UErrorCode errorCode;
+
+    errorCode=U_ZERO_ERROR;
+    csm=ucasemap_open("", 0, &errorCode);
+    if(U_FAILURE(errorCode)) {
+        log_err("ucasemap_open(\"\") failed - %s\n", u_errorName(errorCode));
+        return;
+    }
+
+    iter=ucasemap_getBreakIterator(csm);
+    if(iter!=NULL) {
+        log_err("ucasemap_getBreakIterator() returns %p!=NULL before setting any iterator or titlecasing\n", iter);
+    }
+
+    /* Use default UBreakIterator: Word breaks. */
+    length=ucasemap_toTitle(csm, buffer, LENGTHOF(buffer), beforeTitle, LENGTHOF(beforeTitle), &errorCode);
+    if( U_FAILURE(errorCode) ||
+        length!=LENGTHOF(titleWord) ||
+        0!=u_memcmp(buffer, titleWord, length) ||
+        buffer[length]!=0
+    ) {
+        log_err_status(errorCode, "ucasemap_toTitle(default iterator)=%ld failed - %s\n", (long)length, u_errorName(errorCode));
+    }
+    if (U_SUCCESS(errorCode)) {
+        iter=ucasemap_getBreakIterator(csm);
+        if(iter==NULL) {
+            log_err("ucasemap_getBreakIterator() returns NULL after titlecasing\n");
+        }
+    }
+
+    /* Try U_TITLECASE_NO_BREAK_ADJUSTMENT. */
+    ucasemap_setOptions(csm, U_TITLECASE_NO_BREAK_ADJUSTMENT, &errorCode);
+    if(U_FAILURE(errorCode)) {
+        log_err_status(errorCode, "error: ucasemap_setOptions(U_TITLECASE_NO_BREAK_ADJUSTMENT) failed - %s\n", u_errorName(errorCode));
+        return;
+    }
+
+    length=ucasemap_toTitle(csm, buffer, LENGTHOF(buffer), beforeTitle, LENGTHOF(beforeTitle), &errorCode);
+    if( U_FAILURE(errorCode) ||
+        length!=LENGTHOF(titleWordNoAdjust) ||
+        0!=u_memcmp(buffer, titleWordNoAdjust, length) ||
+        buffer[length]!=0
+    ) {
+        log_err("ucasemap_toTitle(default iterator, no break adjustment)=%ld failed - %s\n", (long)length, u_errorName(errorCode));
+    }
+
+    /* Set a sentence break iterator. */
+    errorCode=U_ZERO_ERROR;
+    sentenceIter=ubrk_open(UBRK_SENTENCE, "", NULL, 0, &errorCode);
+    if(U_FAILURE(errorCode)) {
+        log_err("error: ubrk_open(UBRK_SENTENCE) failed - %s\n", u_errorName(errorCode));
+        ucasemap_close(csm);
+        return;
+    }
+    ucasemap_setBreakIterator(csm, sentenceIter, &errorCode);
+    if(U_FAILURE(errorCode)) {
+        log_err("error: ucasemap_setBreakIterator(sentence iterator) failed - %s\n", u_errorName(errorCode));
+        ubrk_close(sentenceIter);
+        ucasemap_close(csm);
+        return;
+    }
+    iter=ucasemap_getBreakIterator(csm);
+    if(iter!=sentenceIter) {
+        log_err("ucasemap_getBreakIterator() returns %p!=%p after setting the iterator\n", iter, sentenceIter);
+    }
+
+    ucasemap_setOptions(csm, U_TITLECASE_NO_LOWERCASE, &errorCode);
+    if(U_FAILURE(errorCode)) {
+        log_err("error: ucasemap_setOptions(U_TITLECASE_NO_LOWERCASE) failed - %s\n", u_errorName(errorCode));
+        return;
+    }
+
+    /* Use the sentence break iterator with the option. Preflight first. */
+    length=ucasemap_toTitle(csm, NULL, 0, beforeTitle, LENGTHOF(beforeTitle), &errorCode);
+    if( errorCode!=U_BUFFER_OVERFLOW_ERROR ||
+        length!=LENGTHOF(titleSentNoLower)
+    ) {
+        log_err("ucasemap_toTitle(preflight sentence break iterator, no lowercasing)=%ld failed - %s\n", (long)length, u_errorName(errorCode));
+    }
+
+    errorCode=U_ZERO_ERROR;
+    buffer[0]=0;
+    length=ucasemap_toTitle(csm, buffer, LENGTHOF(buffer), beforeTitle, LENGTHOF(beforeTitle), &errorCode);
+    if( U_FAILURE(errorCode) ||
+        length!=LENGTHOF(titleSentNoLower) ||
+        0!=u_memcmp(buffer, titleSentNoLower, length) ||
+        buffer[length]!=0
+    ) {
+        log_err("ucasemap_toTitle(sentence break iterator, no lowercasing)=%ld failed - %s\n", (long)length, u_errorName(errorCode));
+    }
+
+    /* UTF-8 C API coverage. More thorough test via C++ intltest's StringCaseTest::TestCasing(). */
+    {
+        char utf8BeforeTitle[64], utf8TitleSentNoLower[64], utf8[64];
+        int32_t utf8BeforeTitleLength, utf8TitleSentNoLowerLength;
+
+        errorCode=U_ZERO_ERROR;
+        u_strToUTF8(utf8BeforeTitle, (int32_t)sizeof(utf8BeforeTitle), &utf8BeforeTitleLength, beforeTitle, LENGTHOF(beforeTitle), &errorCode);
+        u_strToUTF8(utf8TitleSentNoLower, (int32_t)sizeof(utf8TitleSentNoLower), &utf8TitleSentNoLowerLength, titleSentNoLower, LENGTHOF(titleSentNoLower), &errorCode);
+
+        length=ucasemap_utf8ToTitle(csm, utf8, (int32_t)sizeof(utf8), utf8BeforeTitle, utf8BeforeTitleLength, &errorCode);
+        if( U_FAILURE(errorCode) ||
+            length!=utf8TitleSentNoLowerLength ||
+            0!=uprv_memcmp(utf8, utf8TitleSentNoLower, length) ||
+            utf8[length]!=0
+        ) {
+            log_err("ucasemap_utf8ToTitle(sentence break iterator, no lowercasing)=%ld failed - %s\n", (long)length, u_errorName(errorCode));
+        }
+    }
+
+    ucasemap_close(csm);
+}
+
+#endif
+
 void addCaseTest(TestNode** root);
 
 void addCaseTest(TestNode** root) {
     /* cstrcase.c functions, declared in cucdtst.h */
     addTest(root, &TestCaseLower, "tsutil/cstrcase/TestCaseLower");
     addTest(root, &TestCaseUpper, "tsutil/cstrcase/TestCaseUpper");
-#if !UCONFIG_NO_BREAK_ITERATION
+#if !UCONFIG_NO_BREAK_ITERATION && !UCONFIG_NO_FILE_IO
     addTest(root, &TestCaseTitle, "tsutil/cstrcase/TestCaseTitle");
+    addTest(root, &TestCaseDutchTitle, "tsutil/cstrcase/TestCaseDutchTitle");
 #endif
     addTest(root, &TestCaseFolding, "tsutil/cstrcase/TestCaseFolding");
     addTest(root, &TestCaseCompare, "tsutil/cstrcase/TestCaseCompare");
     addTest(root, &TestUCaseMap, "tsutil/cstrcase/TestUCaseMap");
+#if !UCONFIG_NO_BREAK_ITERATION && !UCONFIG_NO_FILE_IO
+    addTest(root, &TestUCaseMapToTitle, "tsutil/cstrcase/TestUCaseMapToTitle");
+#endif
 }
