@@ -1,19 +1,20 @@
 /********************************************************************
  * COPYRIGHT: 
- * Copyright (c) 1997-2003, International Business Machines Corporation and
+ * Copyright (c) 1997-2013, International Business Machines Corporation and
  * others. All Rights Reserved.
  ********************************************************************/
-/********************************************************************************
+/*****************************************************************************
 *
 * File CCONVTST.C
 *
 * Modification History:
 *        Name                     Description
 *   Madhu Katragadda              7/7/2000        Converter Tests for extended code coverage
-*********************************************************************************
+******************************************************************************
 */
 #include <stdio.h>
-#include "cmemory.h"
+#include <stdlib.h>
+#include <string.h>
 #include "unicode/uloc.h"
 #include "unicode/ucnv.h"
 #include "unicode/utypes.h"
@@ -50,18 +51,22 @@ static UBool testConvertToU( const uint8_t *source, int sourcelen, const UChar *
 
 static void setNuConvTestName(const char *codepage, const char *direction)
 {
-  sprintf(gNuConvTestName, "[Testing %s %s Unicode, InputBufSiz=%d, OutputBufSiz=%d]",
-      codepage,
-      direction,
-      gInBufferSize,
-      gOutBufferSize);
+    sprintf(gNuConvTestName, "[Testing %s %s Unicode, InputBufSiz=%d, OutputBufSiz=%d]",
+        codepage,
+        direction,
+        (int)gInBufferSize,
+        (int)gOutBufferSize);
 }
 
 
 static void TestSurrogateBehaviour(void);
 static void TestErrorBehaviour(void);
+
+#if !UCONFIG_NO_LEGACY_CONVERSION
 static void TestToUnicodeErrorBehaviour(void);
 static void TestGetNextErrorBehaviour(void);
+#endif
+
 static void TestRegressionUTF8(void);
 static void TestRegressionUTF32(void);
 static void TestAvailableConverters(void);
@@ -114,8 +119,12 @@ void addExtraTests(TestNode** root)
 {
      addTest(root, &TestSurrogateBehaviour,         "tsconv/ncnvtst/TestSurrogateBehaviour");
      addTest(root, &TestErrorBehaviour,             "tsconv/ncnvtst/TestErrorBehaviour");
+
+#if !UCONFIG_NO_LEGACY_CONVERSION
      addTest(root, &TestToUnicodeErrorBehaviour,    "tsconv/ncnvtst/ToUnicodeErrorBehaviour");
      addTest(root, &TestGetNextErrorBehaviour,      "tsconv/ncnvtst/TestGetNextErrorBehaviour");
+#endif
+
      addTest(root, &TestAvailableConverters,        "tsconv/ncnvtst/TestAvailableConverters");
      addTest(root, &TestFlushInternalBuffer,        "tsconv/ncnvtst/TestFlushInternalBuffer");
      addTest(root, &TestResetBehaviour,             "tsconv/ncnvtst/TestResetBehaviour");
@@ -131,10 +140,13 @@ static void TestSurrogateBehaviour(){
     {
         UChar sampleText[] = {0x0031, 0xd801, 0xdc01, 0x0032};
         const uint8_t expected[] = {0x31, 0x1a, 0x32};
+
+#if !UCONFIG_NO_LEGACY_CONVERSION
         /*SBCS*/
         if(!convertFromU(sampleText, sizeof(sampleText)/sizeof(sampleText[0]),
                 expected, sizeof(expected), "ibm-920", 0 , TRUE, U_ZERO_ERROR))
             log_err("u-> ibm-920 [UCNV_SBCS] not match.\n");
+#endif
 
         /*LATIN_1*/
         if(!convertFromU(sampleText, sizeof(sampleText)/sizeof(sampleText[0]),
@@ -142,6 +154,8 @@ static void TestSurrogateBehaviour(){
             log_err("u-> LATIN_1 not match.\n");
 
     }
+
+#if !UCONFIG_NO_LEGACY_CONVERSION
     log_verbose("Testing for DBCS and MBCS\n");
     {
         UChar sampleText[]       = {0x00a1, 0xd801, 0xdc01, 0x00a4};
@@ -163,6 +177,7 @@ static void TestSurrogateBehaviour(){
                 expected, sizeof(expected), "ibm-1363", offsets, TRUE, U_ZERO_ERROR))
             log_err("u-> ibm-1363 [UCNV_MBCS] not match.\n");
     }
+
     log_verbose("Testing for ISO-2022-jp\n");
     {
         UChar    sampleText[] =   { 0x4e00, 0x04e01, 0x0031, 0xd801, 0xdc01, 0x0032};
@@ -181,6 +196,7 @@ static void TestSurrogateBehaviour(){
                 expected, sizeof(expected), "iso-2022-jp", offsets , TRUE, U_ZERO_ERROR))
             log_err("u->  not match.\n");
     }
+
     log_verbose("Testing for ISO-2022-cn\n");
     {
         static const UChar    sampleText[] =   { 0x4e00, 0x04e01, 0x0031, 0xd801, 0xdc01, 0x0032};
@@ -210,6 +226,7 @@ static void TestSurrogateBehaviour(){
                 expected, sizeof(expected), "iso-2022-cn", offsets , TRUE, U_ZERO_ERROR))
             log_err("u-> not match.\n");
     }
+
         log_verbose("Testing for ISO-2022-kr\n");
     {
         static const UChar    sampleText[] =   { 0x4e00,0xd801, 0xdc01, 0x04e01, 0x0031, 0xd801, 0xdc01, 0x0032};
@@ -239,6 +256,7 @@ static void TestSurrogateBehaviour(){
                 expected, sizeof(expected), "iso-2022-kr", offsets , TRUE, U_ZERO_ERROR))
             log_err("u-> iso-2022-kr [UCNV_DBCS] not match.\n");
     }
+
         log_verbose("Testing for HZ\n");
     {
         static const UChar    sampleText[] =   { 0x4e00, 0xd801, 0xdc01, 0x04e01, 0x0031, 0xd801, 0xdc01, 0x0032};
@@ -261,11 +279,13 @@ static void TestSurrogateBehaviour(){
         /*hz*/
         if(!convertFromU(sampleText, sizeof(sampleText)/sizeof(sampleText[0]),
                 expected, sizeof(expected), "HZ", 0 , TRUE, U_ZERO_ERROR))
-            log_err("u->  not match.\n");
+            log_err("u-> HZ not match.\n");
         if(!convertFromU(sampleText, sizeof(sampleText)/sizeof(sampleText[0]),
                 expected, sizeof(expected), "HZ", offsets , TRUE, U_ZERO_ERROR))
-            log_err("u->  not match.\n");
+            log_err("u-> HZ not match.\n");
     }
+#endif
+
     /*UTF-8*/
      log_verbose("Testing for UTF8\n");
     {
@@ -294,21 +314,18 @@ static void TestSurrogateBehaviour(){
 
         if(!convertToU(expected, sizeof(expected), 
             sampleText, sizeof(sampleText)/sizeof(sampleText[0]), "UTF8", 0, TRUE, U_ZERO_ERROR ))
-            log_err("UTF8 -> did not match.\n");
+            log_err("UTF8 -> u did not match.\n");
         if(!convertToU(expected, sizeof(expected), 
             sampleText, sizeof(sampleText)/sizeof(sampleText[0]), "UTF8", 0, FALSE, U_ZERO_ERROR ))
-            log_err("UTF8 -> did not match.\n");
+            log_err("UTF8 -> u did not match.\n");
         if(!convertToU(expected, sizeof(expected), 
             sampleText, sizeof(sampleText)/sizeof(sampleText[0]), "UTF8", fromOffsets, TRUE, U_ZERO_ERROR ))
-            log_err("UTF8 -> did not match.\n");
+            log_err("UTF8 ->u  did not match.\n");
         if(!convertToU(expected, sizeof(expected), 
             sampleText, sizeof(sampleText)/sizeof(sampleText[0]), "UTF8", fromOffsets, FALSE, U_ZERO_ERROR ))
-            log_err("UTF8 -> did not match.\n");
+            log_err("UTF8 -> u did not match.\n");
 
     }
-
-
-
 }
 
 /*test various error behaviours*/
@@ -321,6 +338,7 @@ static void TestErrorBehaviour(){
         static const uint8_t expected[] =          { 0x31, 0x1a};
         static const uint8_t expected2[] =         { 0x31, 0x1a, 0x32};
 
+#if !UCONFIG_NO_LEGACY_CONVERSION
         /*SBCS*/
         if(!convertFromU(sampleText, sizeof(sampleText)/sizeof(sampleText[0]),
                 expected, sizeof(expected), "ibm-920", 0, TRUE, U_ZERO_ERROR))
@@ -331,7 +349,7 @@ static void TestErrorBehaviour(){
         if(!convertFromU(sampleText2, sizeof(sampleText2)/sizeof(sampleText2[0]),
                 expected2, sizeof(expected2), "ibm-920", 0, TRUE, U_ZERO_ERROR))
             log_err("u-> ibm-920 [UCNV_SBCS] did not match\n");
-
+#endif
 
         /*LATIN_1*/
         if(!convertFromU(sampleText, sizeof(sampleText)/sizeof(sampleText[0]),
@@ -346,7 +364,7 @@ static void TestErrorBehaviour(){
             log_err("u-> LATIN_1 did not match\n");
     }
 
-
+#if !UCONFIG_NO_LEGACY_CONVERSION
     log_verbose("Testing for DBCS and MBCS\n");
     {
         static const UChar    sampleText[]    = { 0x00a1, 0xd801};
@@ -363,27 +381,23 @@ static void TestErrorBehaviour(){
         static const uint8_t expected3MBCS[] = { 0x01, 0xa2, 0xb4, 0xa1, 0xe0};
         static const int32_t offsets3MBCS[]        = { 0x00, 0x01, 0x01, 0x02, 0x02};
 
-        static const UChar       sampleText4MBCS[] = { 0x0061, 0x00a6, 0xdc01};
+        static const UChar       sampleText4MBCS[] = { 0x0061, 0xFFE4, 0xdc01};
         static const uint8_t expected4MBCS[] = { 0x61, 0x8f, 0xa2, 0xc3, 0xf4, 0xfe};
         static const int32_t offsets4MBCS[]        = { 0x00, 0x01, 0x01, 0x01, 0x02, 0x02 };
 
-
-
-
-
         /*DBCS*/
         if(!convertFromU(sampleText, sizeof(sampleText)/sizeof(sampleText[0]),
                 expectedSUB, sizeof(expectedSUB), "ibm-1363", 0, TRUE, U_ZERO_ERROR))
             log_err("u-> ibm-1363 [UCNV_DBCS portion] is supposed to fail\n");
         if(!convertFromU(sampleText, sizeof(sampleText)/sizeof(sampleText[0]),
-                expected, sizeof(expected), "ibm-1363", 0, FALSE, U_ZERO_ERROR))
+                expected, sizeof(expected), "ibm-1363", 0, FALSE, U_AMBIGUOUS_ALIAS_WARNING))
             log_err("u-> ibm-1363 [UCNV_DBCS portion] is supposed to fail\n");
 
         if(!convertFromU(sampleText, sizeof(sampleText)/sizeof(sampleText[0]),
                 expectedSUB, sizeof(expectedSUB), "ibm-1363", offsetsSUB, TRUE, U_ZERO_ERROR))
             log_err("u-> ibm-1363 [UCNV_DBCS portion] is supposed to fail\n");
         if(!convertFromU(sampleText, sizeof(sampleText)/sizeof(sampleText[0]),
-                expected, sizeof(expected), "ibm-1363", offsets, FALSE, U_ZERO_ERROR))
+                expected, sizeof(expected), "ibm-1363", offsets, FALSE, U_AMBIGUOUS_ALIAS_WARNING))
             log_err("u-> ibm-1363 [UCNV_DBCS portion] is supposed to fail\n");
 
         
@@ -399,7 +413,7 @@ static void TestErrorBehaviour(){
                 expectedSUB, sizeof(expectedSUB), "ibm-1363", 0, TRUE, U_ZERO_ERROR))
             log_err("u-> ibm-1363 [UCNV_MBCS] \n");
         if(!convertFromU(sampleText, sizeof(sampleText)/sizeof(sampleText[0]),
-                expected, sizeof(expected), "ibm-1363", 0, FALSE, U_ZERO_ERROR))
+                expected, sizeof(expected), "ibm-1363", 0, FALSE, U_AMBIGUOUS_ALIAS_WARNING))
             log_err("u-> ibm-1363 [UCNV_MBCS] \n");
 
         if(!convertFromU(sampleText2, sizeof(sampleText2)/sizeof(sampleText2[0]),
@@ -420,12 +434,13 @@ static void TestErrorBehaviour(){
             log_err("u-> ibm-1363 [UCNV_MBCS] \n");
 
         if(!convertFromU(sampleText4MBCS, sizeof(sampleText4MBCS)/sizeof(sampleText4MBCS[0]),
-                expected4MBCS, sizeof(expected4MBCS), "euc-jp", offsets4MBCS, TRUE, U_ZERO_ERROR))
+                expected4MBCS, sizeof(expected4MBCS), "IBM-eucJP", offsets4MBCS, TRUE, U_ZERO_ERROR))
             log_err("u-> euc-jp [UCNV_MBCS] \n");
         if(!convertFromU(sampleText4MBCS, sizeof(sampleText4MBCS)/sizeof(sampleText4MBCS[0]),
-                expected4MBCS, sizeof(expected4MBCS), "euc-jp", offsets4MBCS, FALSE, U_ZERO_ERROR))
+                expected4MBCS, sizeof(expected4MBCS), "IBM-eucJP", offsets4MBCS, FALSE, U_ZERO_ERROR))
             log_err("u-> euc-jp [UCNV_MBCS] \n");
     }
+
     /*iso-2022-jp*/
     log_verbose("Testing for iso-2022-jp\n");
     {
@@ -445,8 +460,8 @@ static void TestErrorBehaviour(){
                 expectedSUB, sizeof(expectedSUB), "iso-2022-jp", offsets, TRUE, U_ZERO_ERROR))
             log_err("u-> iso-2022-jp [UCNV_MBCS] \n");
         if(!convertFromU(sampleText, sizeof(sampleText)/sizeof(sampleText[0]),
-                expected, sizeof(expected), "iso-2022-jp", offsets, FALSE, U_ZERO_ERROR))
-            log_err("u-> ibm-1363 [UCNV_MBCS] \n");
+                expected, sizeof(expected), "iso-2022-jp", offsets, FALSE, U_AMBIGUOUS_ALIAS_WARNING))
+            log_err("u-> iso-2022-jp [UCNV_MBCS] \n");
 
         if(!convertFromU(sampleText2, sizeof(sampleText2)/sizeof(sampleText2[0]),
                 expected2, sizeof(expected2), "iso-2022-jp", offsets2, TRUE, U_ZERO_ERROR))
@@ -465,6 +480,7 @@ static void TestErrorBehaviour(){
                 expected4MBCS, sizeof(expected4MBCS), "iso-2022-jp", offsets4MBCS, FALSE, U_ZERO_ERROR))
             log_err("u-> iso-2022-jp [UCNV_MBCS] \n");
     }
+
     /*iso-2022-cn*/
     log_verbose("Testing for iso-2022-cn\n");
     {
@@ -515,6 +531,7 @@ static void TestErrorBehaviour(){
                 expected4MBCS, sizeof(expected4MBCS), "iso-2022-cn", offsets4MBCS, FALSE, U_ZERO_ERROR))
             log_err("u-> iso-2022-cn [UCNV_MBCS] \n");
     }
+
     /*iso-2022-kr*/
     log_verbose("Testing for iso-2022-kr\n");
     {
@@ -606,10 +623,10 @@ static void TestErrorBehaviour(){
                 expected4MBCS, sizeof(expected4MBCS), "HZ", offsets4MBCS, FALSE, U_ZERO_ERROR))
             log_err("u-> HZ [UCNV_MBCS] \n");
     }
-
-
+#endif
 }
 
+#if !UCONFIG_NO_LEGACY_CONVERSION
 /*test different convertToUnicode error behaviours*/
 static void TestToUnicodeErrorBehaviour()
 {
@@ -619,10 +636,10 @@ static void TestToUnicodeErrorBehaviour()
         const UChar expected[] = { 0x00a1 };
         
         if(!convertToU(sampleText, sizeof(sampleText), 
-                expected, sizeof(expected)/sizeof(expected[0]), "ibm-1363", 0, TRUE, U_ZERO_ERROR ))
+                expected, sizeof(expected)/sizeof(expected[0]), "ibm-1363", 0, TRUE, U_AMBIGUOUS_ALIAS_WARNING ))
             log_err("DBCS (ibm-1363)->Unicode  did not match.\n");
         if(!convertToU(sampleText, sizeof(sampleText), 
-                expected, sizeof(expected)/sizeof(expected[0]), "ibm-1363", 0, FALSE, U_ZERO_ERROR ))
+                expected, sizeof(expected)/sizeof(expected[0]), "ibm-1363", 0, FALSE, U_AMBIGUOUS_ALIAS_WARNING ))
             log_err("DBCS (ibm-1363)->Unicode  with flush = false did not match.\n");
     }
     log_verbose("Testing error conditions for SBCS\n");
@@ -661,6 +678,7 @@ static void TestGetNextErrorBehaviour(){
     }
     ucnv_close(cnv);
 }
+#endif
 
 #define MAX_UTF16_LEN 2
 #define MAX_UTF8_LEN 4
@@ -696,8 +714,42 @@ static void TestRegressionUTF8(){
             log_err("UTF8->Unicode did not match.\n");
         }
     }
+
     free(standardForm);
     free(utf8);
+
+    {
+        static const char src8[] = { (char)0xCC, (char)0x81, (char)0xCC, (char)0x80 };
+        static const UChar expected[] = { 0x0301, 0x0300 };
+        UConverter *conv8;
+        UErrorCode err = U_ZERO_ERROR;
+        UChar pivotBuffer[100];
+        const UChar* const pivEnd = pivotBuffer + 100;
+        const char* srcBeg;
+        const char* srcEnd;
+        UChar* pivBeg;
+
+        conv8 = ucnv_open("UTF-8", &err);
+
+        srcBeg = src8;
+        pivBeg = pivotBuffer;
+        srcEnd = src8 + 3;
+        ucnv_toUnicode(conv8, &pivBeg, pivEnd, &srcBeg, srcEnd, 0, FALSE, &err);
+        if (srcBeg != srcEnd) {
+            log_err("Did not consume whole buffer on first call.\n");
+        }
+
+        srcEnd = src8 + 4;
+        ucnv_toUnicode(conv8, &pivBeg, pivEnd, &srcBeg, srcEnd, 0, TRUE, &err);
+        if (srcBeg != srcEnd) {
+            log_err("Did not consume whole buffer on second call.\n");
+        }
+
+        if (U_FAILURE(err) || (int32_t)(pivBeg - pivotBuffer) != 2 || u_strncmp(pivotBuffer, expected, 2) != 0) {
+            log_err("Did not get expected results for UTF-8.\n");
+        }
+        ucnv_close(conv8);
+    }
 }
 
 #define MAX_UTF32_LEN 1
@@ -734,6 +786,106 @@ static void TestRegressionUTF32(){
     }
     free(standardForm);
     free(utf32);
+
+    {
+        /* Check for lone surrogate error handling. */
+        static const UChar   sampleBadStartSurrogate[] = { 0x0031, 0xD800, 0x0032 };
+        static const UChar   sampleBadEndSurrogate[] = { 0x0031, 0xDC00, 0x0032 };
+        static const uint8_t expectedUTF32BE[] = {
+            0x00, 0x00, 0x00, 0x31,
+            0x00, 0x00, 0xff, 0xfd,
+            0x00, 0x00, 0x00, 0x32
+        };
+        static const uint8_t expectedUTF32LE[] = {
+            0x31, 0x00, 0x00, 0x00,
+            0xfd, 0xff, 0x00, 0x00,
+            0x32, 0x00, 0x00, 0x00
+        };
+        static const int32_t offsetsUTF32[] = {
+            0x00, 0x00, 0x00, 0x00,
+            0x01, 0x01, 0x01, 0x01,
+            0x02, 0x02, 0x02, 0x02
+        };
+
+        if(!convertFromU(sampleBadStartSurrogate, sizeof(sampleBadStartSurrogate)/sizeof(sampleBadStartSurrogate[0]),
+                expectedUTF32BE, sizeof(expectedUTF32BE), "UTF-32BE", offsetsUTF32, TRUE, U_ZERO_ERROR))
+            log_err("u->UTF-32BE\n");
+        if(!convertFromU(sampleBadEndSurrogate, sizeof(sampleBadEndSurrogate)/sizeof(sampleBadEndSurrogate[0]),
+                expectedUTF32BE, sizeof(expectedUTF32BE), "UTF-32BE", offsetsUTF32, TRUE, U_ZERO_ERROR))
+            log_err("u->UTF-32BE\n");
+
+        if(!convertFromU(sampleBadStartSurrogate, sizeof(sampleBadStartSurrogate)/sizeof(sampleBadStartSurrogate[0]),
+                expectedUTF32LE, sizeof(expectedUTF32LE), "UTF-32LE", offsetsUTF32, TRUE, U_ZERO_ERROR))
+            log_err("u->UTF-32LE\n");
+        if(!convertFromU(sampleBadEndSurrogate, sizeof(sampleBadEndSurrogate)/sizeof(sampleBadEndSurrogate[0]),
+                expectedUTF32LE, sizeof(expectedUTF32LE), "UTF-32LE", offsetsUTF32, TRUE, U_ZERO_ERROR))
+            log_err("u->UTF-32LE\n");
+    }
+
+    {
+        static const char srcBE[] = { 0, 0, 0, 0x31, 0, 0, 0, 0x30 };
+        static const UChar expected[] = { 0x0031, 0x0030 };
+        UConverter *convBE;
+        UErrorCode err = U_ZERO_ERROR;
+        UChar pivotBuffer[100];
+        const UChar* const pivEnd = pivotBuffer + 100;
+        const char* srcBeg;
+        const char* srcEnd;
+        UChar* pivBeg;
+
+        convBE = ucnv_open("UTF-32BE", &err);
+
+        srcBeg = srcBE;
+        pivBeg = pivotBuffer;
+        srcEnd = srcBE + 5;
+        ucnv_toUnicode(convBE, &pivBeg, pivEnd, &srcBeg, srcEnd, 0, FALSE, &err);
+        if (srcBeg != srcEnd) {
+            log_err("Did not consume whole buffer on first call.\n");
+        }
+
+        srcEnd = srcBE + 8;
+        ucnv_toUnicode(convBE, &pivBeg, pivEnd, &srcBeg, srcEnd, 0, TRUE, &err);
+        if (srcBeg != srcEnd) {
+            log_err("Did not consume whole buffer on second call.\n");
+        }
+
+        if (U_FAILURE(err) || (int32_t)(pivBeg - pivotBuffer) != 2 || u_strncmp(pivotBuffer, expected, 2) != 0) {
+            log_err("Did not get expected results for UTF-32BE.\n");
+        }
+        ucnv_close(convBE);
+    }
+    {
+        static const char srcLE[] = { 0x31, 0, 0, 0, 0x30, 0, 0, 0 };
+        static const UChar expected[] = { 0x0031, 0x0030 };
+        UConverter *convLE;
+        UErrorCode err = U_ZERO_ERROR;
+        UChar pivotBuffer[100];
+        const UChar* const pivEnd = pivotBuffer + 100;
+        const char* srcBeg;
+        const char* srcEnd;
+        UChar* pivBeg;
+
+        convLE = ucnv_open("UTF-32LE", &err);
+
+        srcBeg = srcLE;
+        pivBeg = pivotBuffer;
+        srcEnd = srcLE + 5;
+        ucnv_toUnicode(convLE, &pivBeg, pivEnd, &srcBeg, srcEnd, 0, FALSE, &err);
+        if (srcBeg != srcEnd) {
+            log_err("Did not consume whole buffer on first call.\n");
+        }
+
+        srcEnd = srcLE + 8;
+        ucnv_toUnicode(convLE, &pivBeg, pivEnd, &srcBeg, srcEnd, 0, TRUE, &err);
+        if (srcBeg != srcEnd) {
+            log_err("Did not consume whole buffer on second call.\n");
+        }
+
+        if (U_FAILURE(err) || (int32_t)(pivBeg - pivotBuffer) != 2 || u_strncmp(pivotBuffer, expected, 2) != 0) {
+            log_err("Did not get expected results for UTF-32LE.\n");
+        }
+        ucnv_close(convLE);
+    }
 }
 
 /*Walk through the available converters*/
@@ -783,6 +935,7 @@ static void TestWithBufferSize(int32_t insize, int32_t outsize){
              log_err("u-> UTF8 did not match.\n");
     }
 
+#if !UCONFIG_NO_LEGACY_CONVERSION
      log_verbose("Testing fromUnicode with UCNV_FROM_U_CALLBACK_ESCAPE  \n");
     {
         UChar inputTest[] = { 0x0061, 0xd801, 0xdc01, 0xd801, 0x0061 };
@@ -798,6 +951,7 @@ static void TestWithBufferSize(int32_t insize, int32_t outsize){
                 (UConverterFromUCallback)UCNV_FROM_U_CALLBACK_ESCAPE, offset,FALSE))
             log_err("u-> ibm-943 with subst with value did not match.\n");
     }
+#endif
 
      log_verbose("Testing fromUnicode for UTF-8 with UCNV_TO_U_CALLBACK_SUBSTITUTE \n");
     {
@@ -811,7 +965,7 @@ static void TestWithBufferSize(int32_t insize, int32_t outsize){
             log_err("utf8->u with substitute did not match.\n");;
     }
 
-
+#if !UCONFIG_NO_LEGACY_CONVERSION
     log_verbose("Testing toUnicode with UCNV_TO_U_CALLBACK_ESCAPE \n");
     /*to Unicode*/
     {
@@ -829,7 +983,7 @@ static void TestWithBufferSize(int32_t insize, int32_t outsize){
             log_err("ibm-943->u with substitute with value did not match.\n");
 
     }
-
+#endif
 }
 
 static UBool convertFromU( const UChar *source, int sourceLen,  const uint8_t *expect, int expectLen, 
@@ -837,13 +991,13 @@ static UBool convertFromU( const UChar *source, int sourceLen,  const uint8_t *e
 {
 
     int32_t i=0;
-    uint8_t *p=0;
+    char *p=0;
     const UChar *src;
-    uint8_t buffer[MAX_LENGTH];
+    char buffer[MAX_LENGTH];
     int32_t offsetBuffer[MAX_LENGTH];
     int32_t *offs=0;
-    uint8_t *targ;
-    uint8_t *targetLimit;
+    char *targ;
+    char *targetLimit;
     UChar *sourceLimit=0;
     UErrorCode status = U_ZERO_ERROR;
     UConverter *conv = 0;
@@ -856,7 +1010,7 @@ static UBool convertFromU( const UChar *source, int sourceLen,  const uint8_t *e
     log_verbose("Converter %s opened..\n", ucnv_getName(conv, &status));
 
     for(i=0; i<MAX_LENGTH; i++){
-        buffer[i]=0xF0;
+        buffer[i]=(char)0xF0;
         offsetBuffer[i]=0xFF;
     }
 
@@ -886,16 +1040,16 @@ static UBool convertFromU( const UChar *source, int sourceLen,  const uint8_t *e
     {
         log_err("Expected %d chars out, got %d FROM Unicode to %s\n", expectLen, targ-buffer, codepage);
         log_verbose("Expected %d chars out, got %d FROM Unicode to %s\n", expectLen, targ-buffer, codepage);
-        printSeqErr((const unsigned char *)buffer, targ-buffer);
+        printSeqErr((const unsigned char *)buffer, (int32_t)(targ-buffer));
         printSeqErr((const unsigned char*)expect, expectLen);
         return FALSE;
     }
 
     if(memcmp(buffer, expect, expectLen)){
         log_err("String does not match. FROM Unicode to codePage%s\n", codepage);
-        printf("\nGot:");
+        log_info("\nGot:");
         printSeqErr((const unsigned char *)buffer, expectLen);
-        printf("\nExpected:");
+        log_info("\nExpected:");
         printSeqErr((const unsigned char *)expect, expectLen);
         return FALSE;
     }
@@ -907,13 +1061,13 @@ static UBool convertFromU( const UChar *source, int sourceLen,  const uint8_t *e
         log_verbose("comparing %d offsets..\n", targ-buffer);
         if(memcmp(offsetBuffer,expectOffsets,(targ-buffer) * sizeof(int32_t) )){
             log_err("did not get the expected offsets. for FROM Unicode to %s\n", codepage);
-            printf("\nGot  : ");
-            printSeqErr((const unsigned char*)buffer, targ-buffer);
+            log_info("\nGot  : ");
+            printSeqErr((const unsigned char*)buffer, (int32_t)(targ-buffer));
             for(p=buffer;p<targ;p++)
-                printf("%d, ", offsetBuffer[p-buffer]); 
-            printf("\nExpected: ");
+                log_info("%d, ", offsetBuffer[p-buffer]); 
+            log_info("\nExpected: ");
             for(i=0; i< (targ-buffer); i++)
-                printf("%d,", expectOffsets[i]);
+                log_info("%d,", expectOffsets[i]);
         }
     }
 
@@ -928,7 +1082,7 @@ static UBool convertToU( const uint8_t *source, int sourceLen, const UChar *expe
     UConverter *conv = 0;
     int32_t i=0;
     UChar *p=0;
-    const uint8_t* src;
+    const char* src;
     UChar buffer[MAX_LENGTH];
     int32_t offsetBuffer[MAX_LENGTH];
     int32_t *offs=0;
@@ -953,7 +1107,7 @@ static UBool convertToU( const uint8_t *source, int sourceLen, const UChar *expe
         offsetBuffer[i]=-1;
     }
 
-    src=source;
+    src=(const char *)source;
     sourceLimit=(uint8_t*)(src+(sourceLen));
     targ=buffer;
     targetLimit=targ+MAX_LENGTH;
@@ -987,26 +1141,26 @@ static UBool convertToU( const uint8_t *source, int sourceLen, const UChar *expe
         if(memcmp(offsetBuffer, expectOffsets, (targ-buffer) * sizeof(int32_t))){
 
             log_err("did not get the expected offsets from %s To UNICODE\n", codepage);
-            printf("\nGot : ");
+            log_info("\nGot : ");
             for(p=buffer;p<targ;p++)
-                printf("%d, ", offsetBuffer[p-buffer]); 
-            printf("\nExpected: ");
+                log_info("%d, ", offsetBuffer[p-buffer]); 
+            log_info("\nExpected: ");
             for(i=0; i<(targ-buffer); i++)
-                printf("%d, ", expectOffsets[i]);
-            printf("\nGot result:");
+                log_info("%d, ", expectOffsets[i]);
+            log_info("\nGot result:");
             for(i=0; i<(targ-buffer); i++)
-                printf("0x%04X,", buffer[i]);
-            printf("\nFrom Input:");
-            for(i=0; i<(src-source); i++)
-                printf("0x%02X,", (unsigned char)source[i]);
-            puts("\n");
+                log_info("0x%04X,", buffer[i]);
+            log_info("\nFrom Input:");
+            for(i=0; i<(src-(const char *)source); i++)
+                log_info("0x%02X,", (unsigned char)source[i]);
+            log_info("\n");
         }
     }
     if(memcmp(buffer, expect, expectLen*2)){
         log_err("String does not match. from codePage %s TO Unicode\n", codepage);
-        printf("\nGot:");
+        log_info("\nGot:");
         printUSeqErr(buffer, expectLen);
-        printf("\nExpected:");
+        log_info("\nExpected:");
         printUSeqErr(expect, expectLen);
         return FALSE;
     }
@@ -1023,16 +1177,16 @@ static UBool testConvertFromU( const UChar *source, int sourceLen,  const uint8_
 {
     UErrorCode status = U_ZERO_ERROR;
     UConverter *conv = 0;
-    uint8_t    junkout[MAX_LENGTH]; /* FIX */
+    char    junkout[MAX_LENGTH]; /* FIX */
     int32_t    junokout[MAX_LENGTH]; /* FIX */
-    uint8_t *p;
+    char *p;
     const UChar *src;
-    uint8_t *end;
-    uint8_t *targ;
+    char *end;
+    char *targ;
     int32_t *offs;
     int i;
     int32_t   realBufferSize;
-    uint8_t *realBufferEnd;
+    char *realBufferEnd;
     const UChar *realSourceEnd;
     const UChar *sourceLimit;
     UBool checkOffsets = TRUE;
@@ -1042,7 +1196,7 @@ static UBool testConvertFromU( const UChar *source, int sourceLen,  const uint8_
     const void* oldContext = NULL;
 
     for(i=0;i<MAX_LENGTH;i++)
-        junkout[i] = 0xF0;
+        junkout[i] = (char)0xF0;
     for(i=0;i<MAX_LENGTH;i++)
         junokout[i] = 0xFF;
 
@@ -1080,7 +1234,7 @@ static UBool testConvertFromU( const UChar *source, int sourceLen,  const uint8_
       checkOffsets = FALSE;
 
     do
-      {
+    {
         end = nct_min(targ + gOutBufferSize, realBufferEnd);
         sourceLimit = nct_min(src + gInBufferSize, realSourceEnd);
 
@@ -1119,11 +1273,11 @@ static UBool testConvertFromU( const UChar *source, int sourceLen,  const uint8_
 
     log_verbose("\nConversion done [%d uchars in -> %d chars out]. \nResult :",
         sourceLen, targ-junkout);
-    if(VERBOSITY)
+    if(getTestOption(VERBOSITY_OPTION))
     {
         char junk[999];
         char offset_str[999];
-        uint8_t *ptr;
+        char *ptr;
 
         junk[0] = 0;
         offset_str[0] = 0;
@@ -1149,9 +1303,9 @@ static UBool testConvertFromU( const UChar *source, int sourceLen,  const uint8_
     {
         log_err("Expected %d chars out, got %d %s\n", expectLen, targ-junkout, gNuConvTestName);
         log_verbose("Expected %d chars out, got %d %s\n", expectLen, targ-junkout, gNuConvTestName);
-        printf("\nGot:");
-        printSeqErr((const unsigned char*)junkout, targ-junkout);
-        printf("\nExpected:");
+        log_info("\nGot:");
+        printSeqErr((const unsigned char*)junkout, (int32_t)(targ-junkout));
+        log_info("\nExpected:");
         printSeqErr((const unsigned char*)expect, expectLen);
         return FALSE;
     }
@@ -1162,7 +1316,7 @@ static UBool testConvertFromU( const UChar *source, int sourceLen,  const uint8_
         if(memcmp(junokout,expectOffsets,(targ-junkout) * sizeof(int32_t) )){
             log_err("did not get the expected offsets. %s", gNuConvTestName);
             log_err("Got  : ");
-            printSeqErr((const unsigned char*)junkout, targ-junkout);
+            printSeqErr((const unsigned char*)junkout, (int32_t)(targ-junkout));
             for(p=junkout;p<targ;p++)
                 log_err("%d, ", junokout[p-junkout]); 
             log_err("\nExpected: ");
@@ -1181,9 +1335,9 @@ static UBool testConvertFromU( const UChar *source, int sourceLen,  const uint8_
     {
         log_err("String does not match. %s\n", gNuConvTestName);
         printUSeqErr(source, sourceLen);
-        printf("\nGot:");
+        log_info("\nGot:");
         printSeqErr((const unsigned char *)junkout, expectLen);
-        printf("\nExpected:");
+        log_info("\nExpected:");
         printSeqErr((const unsigned char *)expect, expectLen);
 
         return FALSE;
@@ -1197,9 +1351,9 @@ static UBool testConvertToU( const uint8_t *source, int sourcelen, const UChar *
     UConverter *conv = 0;
     UChar    junkout[MAX_LENGTH]; /* FIX */
     int32_t    junokout[MAX_LENGTH]; /* FIX */
-    const uint8_t *src;
-    const uint8_t *realSourceEnd;
-    const uint8_t *srcLimit;
+    const char *src;
+    const char *realSourceEnd;
+    const char *srcLimit;
     UChar *p;
     UChar *targ;
     UChar *end;
@@ -1239,7 +1393,7 @@ static UBool testConvertToU( const uint8_t *source, int sourcelen, const UChar *
     }
     /*-------------------------------------*/
 
-    src = source;
+    src = (const char *)source;
     targ = junkout;
     offs = junokout;
     
@@ -1294,7 +1448,7 @@ static UBool testConvertToU( const uint8_t *source, int sourcelen, const UChar *
 
     log_verbose("\nConversion done. %d bytes -> %d chars.\nResult :",
         sourcelen, targ-junkout);
-    if(VERBOSITY)
+    if(getTestOption(VERBOSITY_OPTION))
     {
         char junk[999];
         char offset_str[999];
@@ -1337,7 +1491,7 @@ static UBool testConvertToU( const uint8_t *source, int sourcelen, const UChar *
             for(i=0; i<(targ-junkout); i++)
                 log_err("%X,", junkout[i]);
             log_err("");
-            for(i=0; i<(src-source); i++)
+            for(i=0; i<(src-(const char *)source); i++)
                 log_err("%X,", (unsigned char)source[i]);
         }
     }
@@ -1351,9 +1505,9 @@ static UBool testConvertToU( const uint8_t *source, int sourcelen, const UChar *
     {
         log_err("String does not match. %s\n", gNuConvTestName);
         log_verbose("String does not match. %s\n", gNuConvTestName);
-        printf("\nGot:");
+        log_info("\nGot:");
         printUSeq(junkout, expectlen);
-        printf("\nExpected:");
+        log_info("\nExpected:");
         printUSeq(expect, expectlen); 
         return FALSE;
     }
@@ -1361,6 +1515,7 @@ static UBool testConvertToU( const uint8_t *source, int sourcelen, const UChar *
 
 
 static void TestResetBehaviour(void){
+#if !UCONFIG_NO_LEGACY_CONVERSION
     log_verbose("Testing Reset for DBCS and MBCS\n");
     {
         static const UChar sampleText[]       = {0x00a1, 0xd801, 0xdc01, 0x00a4};
@@ -1398,6 +1553,7 @@ static void TestResetBehaviour(void){
            log_err("ibm-1363 -> did not match.\n");
 
     }
+
     log_verbose("Testing Reset for ISO-2022-jp\n");
     {
         static const UChar    sampleText[] =   { 0x4e00, 0x04e01, 0x0031, 0xd801, 0xdc01, 0x0032};
@@ -1428,6 +1584,7 @@ static void TestResetBehaviour(void){
            log_err("iso-2022-jp -> did not match.\n");
 
     }
+
     log_verbose("Testing Reset for ISO-2022-cn\n");
     {
         static const UChar    sampleText[] =   { 0x4e00, 0x04e01, 0x0031, 0xd801, 0xdc01, 0x0032};
@@ -1471,6 +1628,7 @@ static void TestResetBehaviour(void){
                 offsets1, TRUE))
            log_err("iso-2022-cn -> did not match.\n");
     }
+
         log_verbose("Testing Reset for ISO-2022-kr\n");
     {
         UChar    sampleText[] =   { 0x4e00,0xd801, 0xdc01, 0x04e01, 0x0031, 0xd801, 0xdc01, 0x0032};
@@ -1518,6 +1676,7 @@ static void TestResetBehaviour(void){
                 offsets1, TRUE))
            log_err("iso-2022-kr -> did not match.\n");
     }
+
         log_verbose("Testing Reset for HZ\n");
     {
         static const UChar    sampleText[] =   { 0x4e00, 0xd801, 0xdc01, 0x04e01, 0x0031, 0xd801, 0xdc01, 0x0032};
@@ -1561,6 +1720,8 @@ static void TestResetBehaviour(void){
                 offsets1, TRUE))
            log_err("hz -> did not match.\n");
     }
+#endif
+
     /*UTF-8*/
      log_verbose("Testing for UTF8\n");
     {
@@ -1691,13 +1852,17 @@ TestTruncated() {
         { "UTF-32",     { 0, 0, 0x4e }, 3 },
         { "UTF-32",     { 0xff }, 1 },
         { "UTF-32",     { 0, 0, 0xfe, 0xff, 0 }, 5 },
-
         { "SCSU",       { 0x0e, 0x4e }, 2 }, /* SQU 0x4e */
+
+#if !UCONFIG_NO_LEGACY_CONVERSION
         { "BOCU-1",     { 0xd5 }, 1 },
 
         { "Shift-JIS",  { 0xe0 }, 1 },
 
         { "ibm-939",    { 0x0e, 0x41 }, 2 } /* SO 0x41 */
+#else
+        { "BOCU-1",     { 0xd5 }, 1 ,}
+#endif
     };
     int32_t i;
 
@@ -1731,10 +1896,12 @@ TestUnicodeSet() {
         "SCSU",
         "BOCU-1",
         "CESU-8",
+#if !UCONFIG_NO_LEGACY_CONVERSION
         "gb18030",
+#endif
         "IMAP-mailbox-name"
     };
-
+#if !UCONFIG_NO_LEGACY_CONVERSION
     static const char *const lmbcsNames[]={
         "LMBCS-1",
         "LMBCS-2",
@@ -1749,15 +1916,22 @@ TestUnicodeSet() {
         "LMBCS-18",
         "LMBCS-19"
     };
+#endif
 
     static const NameRange nameRanges[]={
         { "US-ASCII", 0, 0x7f, -1, -1, 0x80, 0x10ffff },
+#if !UCONFIG_NO_LEGACY_CONVERSION
         { "ibm-367", 0, 0x7f, -1, -1, 0x80, 0x10ffff },
+#endif
         { "ISO-8859-1", 0, 0x7f, -1, -1, 0x100, 0x10ffff },
+#if !UCONFIG_NO_LEGACY_CONVERSION
         { "UTF-8", 0, 0xd7ff, 0xe000, 0x10ffff, 0xd800, 0xdfff },
         { "windows-1251", 0, 0x7f, 0x410, 0x44f, 0x3000, 0xd7ff },
-        { "HZ", 0x410, 0x44f, 0x4e00, 0x4eff, 0xac00, 0xd7ff },
+        /* HZ test case fixed and moved to intltest's conversion.txt, ticket #6002 */
         { "shift-jis", 0x3041, 0x3093, 0x30a1, 0x30f3, 0x900, 0x1cff }
+#else
+        { "UTF-8", 0, 0xd7ff, 0xe000, 0x10ffff, 0xd800, 0xdfff }
+#endif
     };
 
     /* open an empty set */
@@ -1769,7 +1943,7 @@ TestUnicodeSet() {
         name=ucnv_getAvailableName(i);
         cnv=ucnv_open(name, &errorCode);
         if(U_FAILURE(errorCode)) {
-            log_err("error: unable to open converter %s - %s\n",
+            log_data_err("error: unable to open converter %s - %s\n",
                     name, u_errorName(errorCode));
             continue;
         }
@@ -1792,7 +1966,7 @@ TestUnicodeSet() {
         name=completeSetNames[i];
         cnv=ucnv_open(name, &errorCode);
         if(U_FAILURE(errorCode)) {
-            log_err("error: unable to open converter %s - %s\n",
+            log_data_err("error: unable to open converter %s - %s\n",
                     name, u_errorName(errorCode));
             continue;
         }
@@ -1809,13 +1983,14 @@ TestUnicodeSet() {
         ucnv_close(cnv);
     }
 
+#if !UCONFIG_NO_LEGACY_CONVERSION
     /* test LMBCS variants which convert all of Unicode except for U+F6xx */
     for(i=0; i<LENGTHOF(lmbcsNames); ++i) {
         errorCode=U_ZERO_ERROR;
         name=lmbcsNames[i];
         cnv=ucnv_open(name, &errorCode);
         if(U_FAILURE(errorCode)) {
-            log_err("error: unable to open converter %s - %s\n",
+            log_data_err("error: unable to open converter %s - %s\n",
                     name, u_errorName(errorCode));
             continue;
         }
@@ -1831,6 +2006,7 @@ TestUnicodeSet() {
 
         ucnv_close(cnv);
     }
+#endif
 
     /* test specific sets */
     for(i=0; i<LENGTHOF(nameRanges); ++i) {
@@ -1864,5 +2040,17 @@ TestUnicodeSet() {
         ucnv_close(cnv);
     }
 
+    errorCode = U_ZERO_ERROR;
+    ucnv_getUnicodeSet(NULL, set, UCNV_ROUNDTRIP_SET, &errorCode);
+    if (errorCode != U_ILLEGAL_ARGUMENT_ERROR) {
+        log_err("error: ucnv_getUnicodeSet(NULL) returned wrong status code %s\n", u_errorName(errorCode));
+    }
+    errorCode = U_PARSE_ERROR;
+    /* Make sure that it does nothing if an error is passed in. Difficult to proper test for. */
+    ucnv_getUnicodeSet(NULL, NULL, UCNV_ROUNDTRIP_SET, &errorCode);
+    if (errorCode != U_PARSE_ERROR) {
+        log_err("error: ucnv_getUnicodeSet(NULL) returned wrong status code %s\n", u_errorName(errorCode));
+    }
+
     uset_close(set);
 }
