@@ -1,5 +1,5 @@
 /***********************************************************************
- * Copyright (c) 1997-2011, International Business Machines Corporation
+ * Copyright (c) 1997-2013, International Business Machines Corporation
  * and others. All Rights Reserved.
  ***********************************************************************/
 
@@ -219,6 +219,8 @@ public: // package
     void Test6703(void);
     void Test3785(void);
     void Test1624(void);
+    void TestIslamicUmAlQura(void);
+    void TestIslamicTabularDates(void);
 
     /**
      * Test the time stamp array recalculation during heavy Calendar usage
@@ -228,6 +230,22 @@ public: // package
      * Test the ISO8601 calendar type
      */
     void TestISO8601(void);
+
+    /**
+     * Test cases for [set|get][Repeated|Skipped]WallTimeOption
+     */
+    void TestAmbiguousWallTimeAPIs(void);
+    void TestRepeatedWallTime(void);
+    void TestSkippedWallTime(void);
+
+    void TestCloneLocale(void);
+
+    /*
+     * utility methods for TestIslamicUmAlQura
+     */
+    void setAndTestCalendar(Calendar* cal, int32_t initMonth, int32_t initDay, int32_t initYear, UErrorCode& status);
+    void setAndTestWholeYear(Calendar* cal, int32_t startYear, UErrorCode& status);
+
 };
 
 #endif /* #if !UCONFIG_NO_FORMATTING */
