@@ -1,6 +1,6 @@
 /***************************************************************************
 *
-*   Copyright (C) 2000-2003, International Business Machines
+*   Copyright (C) 2000-2007, International Business Machines
 *   Corporation and others.  All Rights Reserved.
 *
 ************************************************************************
@@ -20,11 +20,7 @@
 #include "transtst.h"
 #include "transapi.h"
 #include "cpdtrtst.h"
-#include "unhxtrts.h"
-#include "hxuntrts.h"
-#include "ufltlgts.h"
 #include "transrt.h"
-#include "usettest.h"
 #include "jamotest.h"
 #include "trnserr.h"
 #include "reptest.h"
@@ -32,7 +28,7 @@
 #define CASE(id,test) case id:                                \
                           name = #test;                       \
                           if (exec) {                         \
-                              logln(#test "---"); logln("");  \
+                              logln(#test "---"); logln();    \
                               test t;                         \
                               callTest(t, par);               \
                           }                                   \
@@ -45,15 +41,12 @@ void IntlTestTransliterator::runIndexedTest( int32_t index, UBool exec, const ch
         CASE(0, TransliteratorTest);
         CASE(1, TransliteratorAPITest);
         CASE(2, CompoundTransliteratorTest);
-        CASE(3, UniToHexTransliteratorTest);
-        CASE(4, HexToUniTransliteratorTest);
-        CASE(5, TransliteratorRoundTripTest);
-        CASE(6, UnicodeSetTest);
-        CASE(7, JamoTest);
-        CASE(8, TransliteratorErrorTest);
-        CASE(9, ReplaceableTest);
+        CASE(3, TransliteratorRoundTripTest);
+        CASE(4, JamoTest);
+        CASE(5, TransliteratorErrorTest);
+        CASE(6, ReplaceableTest);
 #if !UCONFIG_NO_TRANSLITERATION && defined(U_USE_UNICODE_FILTER_LOGIC_OBSOLETE_2_8)
-        CASE(10, UnicodeFilterLogicTest);
+        CASE(7, UnicodeFilterLogicTest);
 #endif
 
         default: name=""; break;
