@@ -1,6 +1,6 @@
 /********************************************************************
  * COPYRIGHT: 
- * Copyright (c) 1999-2003, International Business Machines Corporation and
+ * Copyright (c) 1999-2013 International Business Machines Corporation and
  * others. All Rights Reserved.
  ********************************************************************/
 /************************************************************************
@@ -63,6 +63,12 @@ public:
 
     void RoundtripRule(const char *dataFile);
 
+    /**
+     * Test creating RuleBasedBreakIterator from RBBIData.
+     **/
+
+    void TestCreateFromRBBIData(void);
+
     /**
      * Tests grouping effect of 'single quotes' in rules.
      **/
@@ -72,6 +78,7 @@ public:
      *  Tests word break status returns.
      */
     void TestRuleStatus();
+    void TestRuleStatusVec();
 
     void TestBug2190();
 
@@ -79,11 +86,13 @@ public:
 
     void TestRegistration();
 
+    void TestRefreshInputText();
+
     /**
      *Internal subroutines
      **/
     /* Internal subroutine used by TestIsBoundary() */ 
-    void doBoundaryTest(RuleBasedBreakIterator& bi, UnicodeString& text, int32_t *boundaries);
+    void doBoundaryTest(BreakIterator& bi, UnicodeString& text, int32_t *boundaries);
 
     /*Internal subroutine used for comparision of expected and acquired results */
     void doTest(UnicodeString& testString, int32_t start, int32_t gotoffset, int32_t expectedOffset, const char* expected);
@@ -91,6 +100,18 @@ public:
 
 };
 
+/**
+ * Special class to enable testing of protected functions in RuleBasedBreakIterator
+ */
+class RBBIWithProtectedFunctions: public RuleBasedBreakIterator {
+public:
+    enum EDontAdopt {
+        kDontAdopt
+    };
+    RBBIWithProtectedFunctions(RBBIDataHeader* data, UErrorCode &status);
+    RBBIWithProtectedFunctions(const RBBIDataHeader* data, enum EDontAdopt dontAdopt, UErrorCode &status);
+};
+
 #endif /* #if !UCONFIG_NO_BREAK_ITERATION */
 
 #endif
