@@ -1,7 +1,7 @@
 /*
  *******************************************************************************
  *
- *   Copyright (C) 1999-2007, International Business Machines
+ *   Copyright (C) 1999-2013, International Business Machines
  *   Corporation and others.  All Rights Reserved.
  *
  *******************************************************************************
@@ -46,8 +46,8 @@ U_CDECL_BEGIN
 
 static void U_CALLCONV ScriptTest(void)
 {
-    if (scriptCodeCount != USCRIPT_CODE_LIMIT) {
-        log_err("ScriptCodes::scriptCodeCount = %n, but UScriptCode::USCRIPT_CODE_LIMIT = %n\n", scriptCodeCount, USCRIPT_CODE_LIMIT);
+    if ((int)scriptCodeCount != (int)USCRIPT_CODE_LIMIT) {
+        log_err("ScriptCodes::scriptCodeCount = %d, but UScriptCode::USCRIPT_CODE_LIMIT = %d\n", scriptCodeCount, USCRIPT_CODE_LIMIT);
     }
 }
 
@@ -385,13 +385,27 @@ static void checkFontVersion(PortableFontInstance *fontInstance, const char *tes
     if (fontChecksum != testChecksum) {
         const char *fontVersionString = fontInstance->getNameString(NAME_VERSION_STRING,
             PLATFORM_MACINTOSH, MACINTOSH_ROMAN, MACINTOSH_ENGLISH);
+        const LEUnicode *uFontVersionString = NULL;
+
+            // The standard recommends that the Macintosh Roman/English name string be present, but
+            // if it's not, try the Microsoft Unicode/English string.
+            if (fontVersionString == NULL) {
+                uFontVersionString = fontInstance->getUnicodeNameString(NAME_VERSION_STRING,
+                    PLATFORM_MICROSOFT, MICROSOFT_UNICODE_BMP, MICROSOFT_ENGLISH);
+            }
 
         log_info("Test %s: this may not be the same font used to generate the test data.\n", testID);
-        log_info("Your font's version string is \"%s\"\n", fontVersionString);
+
+        if (uFontVersionString != NULL) {
+            log_info("Your font's version string is \"%S\"\n", uFontVersionString);
+            fontInstance->deleteNameString(uFontVersionString);
+        } else {
+            log_info("Your font's version string is \"%s\"\n", fontVersionString);
+            fontInstance->deleteNameString(fontVersionString);
+        }
+
         log_info("The expected version string is \"%s\"\n", testVersionString);
         log_info("If you see errors, they may be due to the version of the font you're using.\n");
-
-        fontInstance->deleteNameString(fontVersionString);
     }
 }
 
@@ -399,7 +413,7 @@ static void checkFontVersion(PortableFontInstance *fontInstance, const char *tes
 const char *getSourceTestData() {
     const char *srcDataDir = NULL;
 #ifdef U_TOPSRCDIR
-    srcDataDir = U_TOPSRCDIR U_FILE_SEP_STRING"test"U_FILE_SEP_STRING"testdata"U_FILE_SEP_STRING;
+    srcDataDir = U_TOPSRCDIR U_FILE_SEP_STRING "test" U_FILE_SEP_STRING "testdata" U_FILE_SEP_STRING;
 #else
     srcDataDir = ".."U_FILE_SEP_STRING".."U_FILE_SEP_STRING"test"U_FILE_SEP_STRING"testdata"U_FILE_SEP_STRING;
     FILE *f = fopen(".."U_FILE_SEP_STRING".."U_FILE_SEP_STRING"test"U_FILE_SEP_STRING"testdata"U_FILE_SEP_STRING"rbbitst.txt", "r");
@@ -698,6 +712,7 @@ static void U_CALLCONV DataDrivenTest(void)
 
             delete engine;
 
+            log_verbose("OK - %4d glyphs: %s\n", actual.glyphCount, id);
 free_expected:
             DELETE_ARRAY(expected.positions);
             DELETE_ARRAY(expected.indices);
@@ -977,7 +992,7 @@ static const char *ctest_dataOutDir()
     */
 #if defined (U_TOPBUILDDIR)
     {
-        dataOutDir = U_TOPBUILDDIR "data"U_FILE_SEP_STRING"out"U_FILE_SEP_STRING;
+        dataOutDir = U_TOPBUILDDIR "data" U_FILE_SEP_STRING "out" U_FILE_SEP_STRING;
     }
 #else
 
@@ -1054,6 +1069,11 @@ int main(int argc, char* argv[])
 
     startTime = uprv_getUTCtime();
 
+    if (!initArgs(argc, argv, NULL, NULL)) {
+        /* Error already displayed. */
+        return -1;
+    }
+
     /* Check whether ICU will initialize without forcing the build data directory into
     *  the ICU_DATA path.  Success here means either the data dll contains data, or that
     *  this test program was run with ICU_DATA set externally.  Failure of this check
@@ -1072,7 +1092,11 @@ int main(int argc, char* argv[])
     u_cleanup();
     errorCode = U_ZERO_ERROR;
 
-    /* Initialize ICU */
+    if (!initArgs(argc, argv, NULL, NULL)) {
+        /* Error already displayed. */
+        return -1;
+    }
+/* Initialize ICU */
     ctest_setICU_DATA();    /* u_setDataDirectory() must happen Before u_init() */
     u_init(&errorCode);
 
@@ -1085,7 +1109,7 @@ int main(int argc, char* argv[])
     }
 
     addAllTests(&root);
-    nerrors = processArgs(root, argc, argv);
+    nerrors = runTestRequest(root, argc, argv);
 
     cleanUpTestTree(root);
     u_cleanup();
